// dp[pos] = min cost for to pos
// 1) group positive numbers (not very necessary)
// 2) transform    dp[i] = MAX i*(sum[i]-sum[j])+dp[j]-T   <=>   dp[i] = MAX i*(sum[N]-sum[j])+dp[j] -T-(sum[N]-sum[i])*i
// because the second part of this formula is const for i, we can ignore it and add it to the sum later
// if we denote dp[i]=y, sum[N]-sum[j]=nj, dp[j]=kj, i=x, we can rewrite the formula to y = MAX nj*x+kj
// what we really want is the maximum y for some fixed x over a collection of lines 1..j which can easily be done in logN time using a upper envelope binary tree structute
// however, we don't need to go psycho since in this special case each x we query for equals the previous one +1
// this way we can just maintain a set of lines candidates for the upper envelope, sorted by derivative
// and we want the set to be also sorted by y but reversed (since otherwise we would have lines that will never have a chance of coming on top of the others, which is pointless)
// (note it's the same idea used in team boi 03; bic boi 02; buses ioi 08; batch1(slow) ioi 02 etc.)
// and from this set of candidates, at each iteration we scan them and remove each line which has another line with a higher slope and a higher y at this point
// (for it will never have a chance of coming on top of it and we will never advance from it to some dp[i] in the future)
// and then advance from the line with the smallest slope, since it will be on top of all other lines in the set
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <fstream>
#include <queue>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstdlib>
#include <ctime>


#define SZ size()
#define PB push_back
#define ALL(v) v.begin(),v.end()
#define MP make_pair
#define x first
#define y second

#define LL long long
#define UI unsigned int
#define ULL unsigned long long
#define PI pair<int,int>
#define PD pair<double,double>
#define PLL pair<LL,LL>
#define PULL pair<ULL,ULL>
#define VI vector<int>
#define VD vector<double>
#define VS vector<string>
#define SI set<int>
#define PQ priority_queue
#define VVI vector<vector<int> >
#define IT iterator

#define ABS(x) (((x)>0)?(x):(-(x)))
#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))
#define sign(a) ((a)>0)-((a)<0)
#define sqr(a) ((a)*(a))
#define Repi(n) for (int i=0; i<n; i++)
#define Repj(n) for (int j=0; j<n; j++)
#define Repk(n) for (int k=0; k<n; k++)

#define INF 2000000000
#define EPS 1e-6

#define Time ((double)clock()/CLOCKS_PER_SEC)
#define pause system("pause")

using namespace std;

LL N,T;
#define MAXN 34569
int add[MAXN]; LL sum[MAXN],s[MAXN];
LL dp[MAXN]; int prev[MAXN];
LL days[MAXN];
set<pair<LL,LL> > env;

int main()
{
    scanf("%d%d",&N,&T);   
    Repi(N) { scanf("%d",add+i+1); s[i+1]=s[i]+add[i+1]; }
  
  //marto's optimization - group positive numbers - change "N" with "n", "sum" with "s" and "i" with "days[i]" when used for calculating costs
   int n=0,tsum=0;
    for (int i=1;i<=N;i++)
     {
      tsum+=add[i];
      if (tsum<0)
       s[++n]=tsum,days[n]=i,tsum=0;
     }
    if (days[n]<N)
     s[++n]=tsum,days[n]=N;
    
    for (int i=1;i<=n;i++) s[i]+=s[i-1];

    set<pair<LL,LL> >::IT it,i1,ii;
    pair<set<pair<LL,LL> >::IT,bool> tt;
    dp[0]=0; LL nd;
    LL y1,y2;
    prev[0]=0;
    env.insert(MP(s[n],0));
    for (LL i=1;i<=n;i++)
     {
            i1=env.begin();
            for (it=i1++; i1!=env.end(); it=i1++)
             {
                    y1=it->x*days[i]+dp[it->y],y2=i1->x*days[i]+dp[i1->y];
                    if (y1<=y2) { env.erase(it); if (i1!=env.begin()) i1--; }
             }
             
            prev[i]=env.begin()->y;
            dp[i]=(s[i]-s[prev[i]])*days[i]+dp[prev[i]]-T;
            tt=env.insert(MP(s[n]-s[i],i));
   //         cout<<i<<" -> "; for (it=env.begin(); it!=env.end(); it++) cout<<it->y<<" "; cout<<endl;
     }
    cout<<dp[n]<<"\n";
    printf("%lf\n",Time);
    return 0;
}
