#include <cstdio>
#include <queue>
#include <complex>
using namespace std;

#define X							real()
#define Y							imag()
#define EQ(a, b)					(abs(a - b) < EPS)
#define dot(a, b)					real(conj(a) * (b))
#define cross(a, b)					imag(conj(a) * (b))
#define length(a)					sqrt(dot(a, a))
#define area2(a, b, c)				cross((b) - (a), (c) - (a))
#define online(a, b, c)				EQ(arg((b) - (a)), arg((c) - (b)))

typedef complex<double> point;



const int START = 1;
const int MAX_P = 2048;
const double EPS = 1e-5;

int P, size[MAX_P];
point p[MAX_P];
int a[MAX_P][MAX_P];
bool used[MAX_P][MAX_P];
queue<int> q;

void Read()
{
int E;
	
	scanf("%d %d", & P, & E);
	
	for(int i = 1, x, y; i <= P; i ++)
	{
		scanf("%d %d", & x, & y);
		
		p[i] = point(x, y);
	}
	
	for(int i = 0, n1, n2; i < E; i ++)
	{
		scanf("%d %d", & n1, & n2);
		
		a[n1][size[n1] ++] = n2;
		a[n2][size[n2] ++] = n1;
	}
}

bool can(const double & min_d, int l3, int l2, int l1)
{
	if(length(p[l2] - p[l1]) < min_d - EPS) return false;
	
	if(l3 == 0) return true;
	
	if(EQ(area2(p[l3], p[l2], p[l1]), 0.0)) return online(p[l3], p[l2], p[l1]);
	
	return (0.0 < area2(p[l3], p[l2], p[l1]));
}

bool rec(const double & min_d, int last, int now)
{
	used[last][now] = true;
	
	if(now == START && last) return true;
	
	for(int i = 0; i < size[now]; i ++)
	{
		int next = a[now][i];
		
		if(!used[now][next] && can(min_d, last, now, next))
		{
			q.push(next);
			
			if(rec(min_d, now, next)) return true;
			
			q.pop();
		}
	}
	
	return false;
}

void Solve()
{
double low = 0.0, mid, high = 1e5;
	
	for(q.push(START); EPS < high - low;)
	{
		mid = (low + high) * 0.5;
		
		memset(used, false, sizeof(used));
		
		if(rec(mid, 0, START)) low = mid;
		else high = mid;
		
		while(!q.empty()) q.pop();
		q.push(START);
	}
	
	printf("\nDistance: %.4lf\n", low);
	
	memset(used, false, sizeof(used));
	
	rec(low, 0, START);
}

void Write()
{
	printf("%d\n", q.size());
	
/*
	for(; !q.empty(); q.pop())
	{
		printf("%d ", q.front());
	}
	printf("\n");
*/
	
	printf("DFS RUN TIME: %.5lf\n", clock() / double(CLOCKS_PER_SEC));
	
//	system("pause");
}

int main()
{
	Read();
	
	Solve();
	
	Write();
	
	return 0;
}
/*
TEST 0:
5 6
1 0
2 1
1 1
0 1
1 2
1 2
2 5
1 4
5 4
2 3
4 3
*/
