// see what happens when the tree is a chain - for, example, for N=9, a good labeling is: 5 4 6 3 7 2 8 1 9 - you put the middle number in the middle and 
//then alternate puting either the first larger or the first smaller number which has not yet been put, that way we're sure that the difference between consequtive nodes is strictly increasing
//the upgrade to a caterpillar can be illustrated like this - if you've labeled a node with a larger number, you put a small label not only to it's next neighbour on the chain,
//but to all it's unlabeled neighbours as well - that way we make sure that its differences with all neighnours will be unique while at the same time not interrupting
//the increasing differences logic
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <fstream>
#include <queue>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstdlib>
#include <ctime>


#define SZ size()
#define PB push_back
#define ALL(v) v.begin(),v.end()
#define MP make_pair
#define x first
#define y second

#define LL long long
#define UI unsigned int
#define ULL unsigned long long
#define PI pair<int,int>
#define PD pair<double,double>
#define PLL pair<LL,LL>
#define PULL pair<ULL,ULL>
#define VI vector<int>
#define VD vector<double>
#define VS vector<string>
#define SI set<int>
#define PQ priority_queue
#define VVI vector<vector<int> >
#define IT iterator

#define ABS(x) (((x)>0)?(x):(-(x)))
#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))
#define sign(a) ((a)>0)-((a)<0)
#define sqr(a) ((a)*(a))
#define Repi(n) for (int i=0; i<n; i++)
#define Repj(n) for (int j=0; j<n; j++)
#define Repk(n) for (int k=0; k<n; k++)

#define INF 2000000000
#define EPS 1e-6

#define Time ((double)clock()/CLOCKS_PER_SEC)
#define pause system("pause")

using namespace std;
#define MAXN 10010
int N;
VI adj[MAXN];
int dist[MAXN],next[MAXN],prev[MAXN],farthest,start,end,small,large,label[MAXN];

void dfs(int v)
{
    if (dist[v]>dist[farthest]) farthest=v;
    Repi(adj[v].SZ)
     if (dist[adj[v][i]]==-1)
      {
            dist[adj[v][i]]=dist[v]+1;
            prev[adj[v][i]]=v;
            dfs(adj[v][i]);
      }
}


int main()
{
    int a,b;
    scanf("%d",&N);
    Repi(N-1)
     {
            scanf("%d%d",&a,&b);
            adj[a].PB(b); adj[b].PB(a);
     }
    
    //find chain (farthest route in tree)
    memset(dist,-1,sizeof(dist));
    dist[1]=0;
    farthest=1;
    dfs(1);
    memset(dist,-1,sizeof(dist));
    start=farthest;
    dist[farthest]=0;
    prev[start]=-1;
    dfs(start);
    end=farthest;

    //count how many "smaller" and how many "larger" numbers we need
    int v=prev[end];
    bool sm=0;
    small=0,large=1;
    while (v>-1)
       {
         if (sm) large+=adj[v].SZ-1;
         else small+=adj[v].SZ-1;
         sm=!sm;
         v=prev[v];
       };
    
    //label nodes
    next[end]=-1;
    v=end; label[v]=small+1;
    sm=0;
    while (v>-1)
      {
            if (prev[v]>-1) next[prev[v]]=v;
            
            if (sm)
             {
              Repi(adj[v].SZ)
               if (adj[v][i]!=next[v])
                  label[adj[v][i]]=small--;
              if (prev[v]>-1)
               {
                 int tt;
                 Repi(adj[v].SZ)
                  if (label[adj[v][i]]==small+1) { tt=adj[v][i]; break; }
                 swap(label[tt],label[prev[v]]);
               }
             }
            else
             {
              Repi(adj[v].SZ)
               if (adj[v][i]!=next[v])
                  label[adj[v][i]]=N-(--large);
              if (prev[v]>-1)
               {
                 int tt;
                 Repi(adj[v].SZ)
                  if (label[adj[v][i]]==N-large) { tt=adj[v][i]; break; }
                 swap(label[tt],label[prev[v]]);
               }
             }
           sm=!sm;
           v=prev[v];
      }
    
    //check if label differences are really unique
    cout<<"if something else besides the answer (and this message) appears, there's an error ;) however, that's not very likely...\n";
    set<int> used;
    for (int i=1;i<=N;i++)
     Repj(adj[i].SZ)
      if (adj[i][j]>i)
       {
         int dif=abs(label[i]-label[adj[i][j]]);
         if (used.find(dif)!=used.end())
          cout<<i<<" - "<<adj[i][j]<<" = "<<dif<<" ("<<label[i]<<" "<<label[adj[i][j]]<<")\n";
         used.insert(dif);
       }

    //print
    for (int i=1;i<N;i++) printf("%d ",label[i]); printf("%d\n",label[N]);
    
    return 0;
}
