// count with bit (i crosses all with X<Xi && V>Vi) ; then perform simplified bentley-ottoman (T is the abscissa, X is the ordinate) to find intersection points
// instead of using a set, since we don't add nor delete lines, just keep them sorted and swap consequtive lines when intersecting
// note: use PQ instead of set - set is slow; to avoid duplicating intersection points, keep events strictly sorted in queue and check whether the next pair of intersecting lines is the same as the last one
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <fstream>
#include <queue>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstdlib>
#include <ctime>


#define SZ size()
#define PB push_back
#define ALL(v) v.begin(),v.end()
#define MP make_pair
#define x first
#define y second

#define LL long long
#define UI unsigned int
#define ULL unsigned long long
#define PI pair<int,int>
#define PD pair<double,double>
#define PLL pair<LL,LL>
#define PULL pair<ULL,ULL>
#define VI vector<int>
#define VD vector<double>
#define VS vector<string>
#define SI set<int>
#define PQ priority_queue
#define VVI vector<vector<int> >
#define IT iterator

#define ABS(x) (((x)>0)?(x):(-(x)))
#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))
#define sign(a) ((a)>0)-((a)<0)
#define sqr(a) ((a)*(a))
#define Repi(n) for (int i=0; i<n; i++)
#define Repj(n) for (int j=0; j<n; j++)
#define Repk(n) for (int k=0; k<n; k++)

#define INF 2000000000
#define EPS 1e-6

#define Time ((double)clock()/CLOCKS_PER_SEC)
#define pause system("pause")

using namespace std;

int n,m,N,M,P,K;
#define F(s,n) for (int i=(s);i<(n);i++)
#define MAXN 560000
#define MOD 1000000
int X[MAXN],V[MAXN];

int tree[256];
inline void push(int v)
{
    for(;v;v-=v&(-v))
     tree[v]++;
}

inline int query(int v)
{
    int res=0;
    for (;v<=256;v+=v&(-v))
     res=res+tree[v];
    return res;
}

int prev[MAXN],next[MAXN];
struct tip{ 
    double x,t; int a,b; 
    tip(){}; 
    tip(double _x,double _t,int _a,int _b){x=_x,t=_t,a=_a,b=_b;};
bool operator<(const tip &A) const
 {   return (A.t==t)?(x==A.x)?(a>A.a)?b>A.b:a>A.a:x>A.x:t>A.t;    };
 };

PQ<tip> q;
double T;

inline void addevent(int a,int b)
{
    if (a<=0 || b<=0 || a>N || b>N) return;
    double t=(double)(X[a]-X[b])/(double)(V[b]-V[a]);
    double x=(double)X[a]+(double)V[a]*t;
    if (t<T) return;
    q.push(tip(x,t,a,b));
}

int main()
{
    scanf("%d",&N);
    int ans=0;
    F(1,N+1)
     {
            scanf("%d%d",X+i,V+i);
            push(V[i]);
            ans=(ans+query(V[i]+1))%MOD;
     }
    printf("%d\n",ans);
    
    F(1,N+1)
     {        prev[i]=i-1,next[i]=i+1;  addevent(i,i+1);   } 
    
    int crossings=0; tip tmp; int a,b,pa,pb,na,nb;
    int lasta=-1,lastb=-1;
    while (!q.empty() && crossings<10000)
     {
            tmp=q.top(); q.pop();
            a=tmp.a,b=tmp.b; T=tmp.t;
            if (a==lasta&&b==lastb) continue;
            lasta=a,lastb=b;
            printf("%d %d\n",a,b);
            
            pa=prev[a],na=next[a],pb=prev[b],nb=next[b];
            prev[a]=b; next[b]=a; next[pa]=b;
            prev[b]=pa; next[a]=nb; prev[nb]=a;
            addevent(a,next[a]); addevent(prev[b],b);
            crossings++;
     }
    cout<<Time<<endl;
    return 0;
}
