//we can obtain the total volume that we would have to remove from each pile by solving a linear system of equations where
//coef[i][j] represents what portion of the total volume that 'j' will spread will go to 'i'
//and coef[i][N] (the results) is the difference between the desired volume of 'i' (=the average of all initial volumes) and its inital volume
//it is obvious that if we substitute each x[i] with the desired value, coef[j][0]*x[0] + coef[j][1]*x[1] ... + coef[j][N-1]*x[N-1] will equal coef[j][N] for all j
//now all that is left is to spread the desired volume from each pile in such a way that at any moment all volumes are >= 0 and the total number of moves is minimal
//one possible solution is to greedily pick the pile that has the largest volume at the moment and remove as much from it as we can, which yields pretty good results so far :)
//see comments in implementation for clarification on details
#include <iostream> 
#include <fstream>
#include <sstream>
#include <cmath>  
#include <ctime> 
#include <cstdlib>
#include <cstdio>
#include <vector>   
#include <set>
#include <map>
#include <bitset>
#include <queue>
#include <iomanip>
#include <algorithm>
#include <utility>
#include <numeric>
#include <functional>

using namespace std;

#define Repi(n) for (int i=0;i<(n);i++)
#define Repj(n) for (int j=0;j<(n);j++)
#define Repk(n) for (int k=0;k<(n);k++)
#define PB push_back
#define MP make_pair
#define x first
#define y second
#define ALL(v) (v).begin(),(v).end()
#define SZ size()
#define sqr(a) ((a)*(a))
#define LL long long
#define PI pair<int,int>
#define PD pair<double,double>
#define VI vector<int>


#define MAXN 256

//after solving it using double and testing, we can see that all coefficients remain integers at all times (proof?)
int a[MAXN],sum,N;
int c[MAXN][MAXN];
int x[MAXN];

void print()
{
   Repi(N)
    {
      cout<<"          ";
      Repj(N+1)
       printf("%6.2lf",c[i][j]);
      cout<<endl;
    }
   cout<<endl;
}


double res[MAXN];
void gauss()
{
  int coef;
  
  //gauss
  Repi(N-1)
   {
      for (int j=i+1;j<N;j++)
       {
          coef=-c[j][i]/c[i][i];
          //this part can be optimized, but who cares...
          for (int k=i;k<=N;k++)
           c[j][k]+=c[i][k]*coef;
       }
   }
   
  //get x's
  int minx=0;
  Repi(N) res[i]=c[i][N];
  for (int i=N-1;i>=0;i--)
   {
      if (c[i][i] && c[i][N]) x[i]=c[i][N]/c[i][i]; else x[i]=0;
      if (x[minx]>x[i]) minx=i;
      for (int j=i-1;j>=0;j--)
       c[j][N]-=c[j][i]*x[i];
   }
  
  //offset so that all x > 0 (because the last x is undefined)
  if (x[minx]>=0) return;
  double offs=-x[minx];
  Repi(N) x[i]+=offs; return;
  
  //same as incrementing all x with offs (proof?)
  /*
  cout<<offs<<": "; Repi(N) cout<<res[i]<<" "; cout<<endl;
  x[minx]=0;
     for (int i=minx;i<N-1;i++)
      {
         if(res[i] && c[i][i+1]) x[i+1]=res[i]/c[i][i+1]; else x[i+1]=0;
         cout<<"   new x "<<i+1<<" = "<<x[i+1]<<endl;
         res[i+1]-=c[i+1][i+1]*x[i+1];
      }
     for (int i=minx-1;i>=0;i--)
      {
        if (res[i] && c[i][i]) x[i]=res[i]/c[i][i]; else x[i]=0;
        cout<<"   new x "<<i<<" = "<<x[i]<<endl;
        if (i) res[i-1]-=c[i-1][i]*x[i];
      }
  */
}




int l[MAXN];

struct cmp
{
   bool operator()(const int v,const int w) const
    { return (a[v]==a[w])?v>w:a[v]>a[w];   } //better cmp ?
};

vector<PI> ans;
void solve()
{
   Repi(N) { l[i]=(int)x[i]; if (i && i<N-1) l[i]<<=1; }
   set<int,cmp> q;
   Repi(N) if (l[i]) q.insert(i);
   ans.clear();
   int takefrom,take;
   while (!q.empty())
    {
          takefrom=*q.begin();
          take=min(l[takefrom],a[takefrom]);
          if ((take&1) && takefrom && takefrom<N-1) take^=1;
          ans.PB(MP(takefrom,take>>(takefrom && takefrom<N-1)));
          q.erase(q.begin());
          if (takefrom) q.erase(takefrom-1);
          if (takefrom<N-1) q.erase(takefrom+1);  //fist remove...
          
          l[takefrom]-=take; a[takefrom]-=take;   //..then modify...
          if (takefrom && takefrom<N-1) take>>=1;
          if (takefrom) { a[takefrom-1]+=take; if (l[takefrom-1]) q.insert(takefrom-1); }   //...and finally insert again (the order is rather important)
          if (takefrom<N-1) { a[takefrom+1]+=take; if (l[takefrom+1]) q.insert(takefrom+1); }
          if (l[takefrom]) q.insert(takefrom);
    }
    
   printf("%d\n",ans.SZ);
   Repi(ans.SZ)
    printf("%d %d\n",ans[i].x+1,ans[i].y);
}



int real[MAXN];
bool check()
{
    Repi(ans.SZ)
     {
       int from=ans[i].x,take=ans[i].y;
       if (from) real[from-1]+=take,real[from]-=take;
       if (from<N-1) real[from+1]+=take,real[from]-=take;
       if (real[from]<0)
        {
           fprintf(stderr,"!!!!!! %d got %d after step %d!!!!\n",from,real[from],i+1);
           return 0;
        }
     }
    Repi(N) 
     if (real[i]!=sum)
      {
         fprintf(stderr,"!!!! %d has %d instead of %d after all steps!!\n",i,real[i],sum);
         return 0;
      }
     
    return 1;
}

int main()
{
   scanf("%d",&N);
   sum=0;
   Repi(N) scanf("%d",a+i),sum+=a[i];
   memcpy(real,a,sizeof(real));
   sum/=N;
   
   memset(c,0,sizeof(c));
   Repi(N)
    {
      if (i) c[i][i-1]=1;
      if (i<N-1) c[i][i+1]=1;
      c[i][i]=-2;
      c[i][N]=sum-a[i];
    }
   c[0][0]=c[N-1][N-1]=-1.;
   
   gauss();
   solve();
   
   if (check())
    fprintf(stderr,"solution is CORRECT :)\n");
   else
    fprintf(stderr,"solution is WRONG :~(\n");
   fprintf(stderr,"time: %.3lf\n",(double)clock()/CLOCKS_PER_SEC);
   return 0;
}
