#include <iostream>
#include <fstream>
#include <cstdlib>
using namespace std;

const int WT_LEFT = 0;
const int WT_RIGHT = 1;
const int WT_UP = 2;
const int WT_DOWN = 3;

const int MAX_N = 10;
const int MAX_MOVES = 64;

int offsetWalls[4] = {WT_UP,WT_DOWN,WT_LEFT,WT_RIGHT};
int offset[4][2] = {{-1,0},{1,0},{0,-1},{0,1}};

int gN;
int gWappoRow;
int gWappoCol;
int gM1Row;
int gM2Row;
int gM1Col;
int gM2Col;
int gExitRow;
int gExitCol;
int gMatrix[MAX_N][MAX_N];
int gMoves[MAX_MOVES];
int gMovesCount;
bool gTrap[MAX_N][MAX_N];

bool isOutsideBoard(int aRow,int aCol)
{
	return (aRow<1 || aCol<1 || aRow>gN || aCol>gN);
}


bool isTrap(int aRow,int aCol) 
{
	if (!isOutsideBoard(aRow,aCol))
	{
		return gTrap[aRow][aCol];
	} else
	{
		cout<<"Internal Error 1"<<endl;
		exit(1);
	}
	
}

bool isPossible(int aRow,int aCol) 
{
	if (aRow==gExitRow && aCol==gExitCol)
	{
		return true;
	}		
	if (isOutsideBoard(aRow,aCol)) 
	{
		return false;
	}
	return (!isTrap(aRow,aCol));
}


bool hasWall(int aRow,int aCol,int aWallType)
{
	switch (aWallType)
	{
	case WT_LEFT:
		return (gMatrix[aRow][aCol]&1)!=0;
	case WT_RIGHT:
		return (gMatrix[aRow][aCol]&2)!=0;
	case WT_UP:
		return (gMatrix[aRow][aCol]&4)!=0;
	case WT_DOWN:
		return (gMatrix[aRow][aCol]&8)!=0;
	}
	cout<<"Internal Error 2"<<endl;
	exit(1);	
}

void moveMonster(int aWappoRow,
				 int aWappoCol,
				 int &aMRow,
				 int &aMCol)
{

	if (aMRow==aWappoRow && aMCol==aWappoCol)
	{
		return;
	}

	if (aMRow==aWappoRow)
	{
		if (aWappoCol<aMCol)
		{
			if (!hasWall(aMRow,aMCol,WT_LEFT))
			{
				aMCol--;
			}
		} else
		{
			if (!hasWall(aMRow,aMCol,WT_RIGHT))
			{
				aMCol++;
			}
		}
		return;
	}
	if (aMCol==aWappoCol)
	{
		if (aWappoRow<aMRow)
		{
			if (!hasWall(aMRow,aMCol,WT_UP))
			{
				aMRow--;
			}
		} else
		{
			if (!hasWall(aMRow,aMCol,WT_DOWN))
			{
				aMRow++;
			}
		}
		return;
	}

	if (aWappoRow<aMRow && aWappoCol<aMCol)
	{
		if (!hasWall(aMRow,aMCol,WT_LEFT))
		{
			aMCol--;
		} else 
		{
			if (!hasWall(aMRow,aMCol,WT_UP))
			{
				aMRow--;
			}
		}
		return;
	}


	if (aWappoRow<aMRow && aWappoCol>aMCol)
	{
		if (!hasWall(aMRow,aMCol,WT_RIGHT))
		{
			aMCol++;
		} else 
		{
			if (!hasWall(aMRow,aMCol,WT_UP))
			{
				aMRow--;
			}
		}
		return;
	}

	if (aWappoRow>aMRow && aWappoCol>aMCol)
	{
		if (!hasWall(aMRow,aMCol,WT_RIGHT))
		{
			aMCol++;
		} else 
		{
			if (!hasWall(aMRow,aMCol,WT_DOWN))
			{
				aMRow++;
			}
		}
		return;
	}

	if (aWappoRow>aMRow && aWappoCol<aMCol)
	{
		if (!hasWall(aMRow,aMCol,WT_LEFT))
		{
			aMCol--;
		} else 
		{
			if (!hasWall(aMRow,aMCol,WT_DOWN))
			{
				aMRow++;
			}
		}
		return;
	}
}

void moveMonsters(int aWappoRow,
					 int aWappoCol,
					 int &aM1Row,
					 int &aM1Col,
					 int &aM2Row,
					 int &aM2Col,
					 int &aM1TrapLeft,
					 int &aM2TrapLeft,
					 bool aAreUnited) 
{

	if (aAreUnited)
	{
		moveMonster(aWappoRow,aWappoCol,aM1Row,aM1Col);
		aM2Row = aM1Row;
		aM2Col = aM1Col;
	} else 
	{
			if (aM1TrapLeft>0) 
			{
				aM1TrapLeft--;
			} else 
			{
				int aTempM1Row;
				int aTempM1Col;

				aTempM1Row = aM1Row;
				aTempM1Col = aM1Col;
				moveMonster(aWappoRow,aWappoCol,aM1Row,aM1Col);
				if ((aTempM1Row!=aM1Row || aTempM1Col!=aM1Col) && isTrap(aM1Row,aM1Col))
				{
					aM1TrapLeft = 3;
				}
			}
			if (aM2TrapLeft>0) 
			{
				aM2TrapLeft--;
			} else 
			{
				int aTempM2Row;
				int aTempM2Col;

				aTempM2Row = aM2Row;
				aTempM2Col = aM2Col;
				moveMonster(aWappoRow,aWappoCol,aM2Row,aM2Col);
				if ((aTempM2Row!=aM2Row || aTempM2Col!=aM2Col) && isTrap(aM2Row,aM2Col))
				{
					aM2TrapLeft = 3;
				}
			}
	}

}

void simulateGame(bool aBetterSolution)
{
	int M1TrapLeft = 0;
	int M2TrapLeft = 0;

	for (int i=0;i<gMovesCount;i++)
	{
		if (gMoves[i]<0 || gMoves[i]>3) 
		{
			cout<<0<<endl<<"Invalid move "<<i<<endl;
			return;
		}

		if (hasWall(gWappoRow,gWappoCol,offsetWalls[gMoves[i]]))
		{
			cout<<0<<endl<<"Invalid move "<<i<<endl<<"Heroes cannot pass through walls"<<endl;
			return;

		}

		gWappoRow+=offset[gMoves[i]][0];
		gWappoCol+=offset[gMoves[i]][1];

		if (!isPossible(gWappoRow,gWappoCol))
		{
			cout<<0<<endl<<"After "<<i<<" moves the hero stepped on a wrong square"<<endl;
			return;
		}

		if (gWappoRow==gExitRow && gWappoCol==gExitCol)
		{
			break;
		}

		if ((gWappoRow==gM1Row && gWappoCol==gM1Col) || (gWappoRow==gM2Row && gWappoCol==gM2Col))
		{
			cout<<0<<endl<<"Hero is captured after "<<i<<" moves!"<<endl;
			return;
		}

		if (gM1Row==gM2Row && gM1Col==gM2Col)
		{
				// the megamonster has three moves
				for (int j=0;j<3;j++)
				{
					moveMonsters(gWappoRow,gWappoCol,gM1Row,gM1Col,gM2Row,gM2Col,M1TrapLeft,M2TrapLeft,true);

					// if wappo is caught by the monsters
					if ((gWappoRow==gM1Row && gWappoCol==gM1Col) || (gWappoRow==gM2Row && gWappoCol==gM2Col))
					{
						cout<<0<<endl<<"Hero is captured after "<<i<<" moves!"<<endl;
						return;
					}

				}
		} 
		else
		{
			// the two monsters are not united

			if (M1TrapLeft>0 && M2TrapLeft>0)
			{
				M1TrapLeft--;
				M2TrapLeft--;
			}
			else
			{

					if (M1TrapLeft>0 || M2TrapLeft>0)
					{
						if (M1TrapLeft>0)
						{
							M1TrapLeft--;
							for (int j=0;j<2;j++)
							{
								int saveMRow;
								int saveMCol;

								saveMRow = gM2Row; saveMCol = gM2Col;
								moveMonster(gWappoRow,gWappoCol,gM2Row,gM2Col);

								// if wappo is captured no next move exists
								if (gM2Row==gWappoRow && gM2Col==gWappoCol)
								{
									cout<<0<<endl<<"Hero is captured after "<<i<<" moves!"<<endl;
								}

								// if the two monsters are united, they do not make a next move
								if (gM1Row==gM2Row && gM1Col==gM2Col) 
								{
									M1TrapLeft = 0;
									M2TrapLeft = 0;
									break;
								}

								// if the monster is trapped it does not make a next move
								if (isTrap(gM2Row,gM2Col))
								{
									if (saveMRow!=gM2Row || saveMCol!=gM2Col)
									{
										M2TrapLeft = 3;
									}
									break;
								}

							}

						} else
						{
							M2TrapLeft--;
							for (int j=0;j<2;j++)
							{
								int saveMRow;
								int saveMCol;

								saveMRow = gM1Row; saveMCol = gM1Col;

								moveMonster(gWappoRow,gWappoCol,gM1Row,gM1Col);

								// if wappo is captured no next move exists
								if (gM1Row==gWappoRow && gM1Col==gWappoCol)
								{
										cout<<0<<endl<<"Hero is captured after "<<i<<" moves!"<<endl;
										return;
								}

								// if the two monsters are united, they do not make a next move
								if (gM1Row==gM2Row && gM1Col==gM2Col) 
								{
									M1TrapLeft = 0;
									M2TrapLeft = 0;
									break;
								}

								// if the monster is trapped it does not make a next move
								if (isTrap(gM1Row,gM1Col))
								{
									if (saveMRow!=gM1Row || saveMCol!=gM1Col)
									{
										M1TrapLeft = 3;
									}
									break;
								}

							}
						}
					}
					else
					{
						// both monsters are not trapped
						for (int j=0;j<2;j++) 
						{
							if (M1TrapLeft>0 || M2TrapLeft>0)
							{
								if (M2TrapLeft==0)
								{
									int saveMRow;
									int saveMCol;
									saveMRow = gM2Row;
									saveMCol = gM2Col;
									moveMonster(gWappoRow,gWappoCol,gM2Row,gM2Col);
									if (isTrap(gM2Row,gM2Col))
									{
										if (saveMRow!=gM2Row || saveMCol!=gM2Col)
										{
											M2TrapLeft = 3;
										}
									}
								}
								if (M1TrapLeft==0)
								{
									int saveMRow;
									int saveMCol;
									saveMRow = gM1Row;
									saveMCol = gM1Col;
									moveMonster(gWappoRow,gWappoCol,gM1Row,gM1Col);
									if (isTrap(gM1Row,gM1Col))
									{
										if (saveMRow!=gM1Row || saveMCol!=gM1Col)
										{
											M1TrapLeft = 3;
										}
									}

									moveMonster(gWappoRow,gWappoCol,gM1Row,gM1Col);
								}
							} else
							{
								moveMonsters(gWappoRow,gWappoCol,gM1Row,gM1Col,gM2Row,gM2Col,M1TrapLeft,M2TrapLeft,false);
							}

							// if wappo is captured no next move exists
							if ((gM1Row==gWappoRow && gM1Col==gWappoCol) || (gM2Row==gWappoRow && gM2Col==gWappoCol))
							{
								cout<<0<<endl<<"Hero is captured after "<<i<<" moves!"<<endl;
								return;
							}

							// if the two monsters are united, they do not make a next move
							if (gM1Row==gM2Row && gM1Col==gM2Col) 
							{
								M1TrapLeft = 0;
								M2TrapLeft = 0;

								break;
							}
						}

					}

			}

		}

	}

	if (gWappoRow==gExitRow && gWappoCol==gExitCol)
	{
		if (aBetterSolution)
		{
			cout<<5<<endl<<"Ooooops! Contestant found a better solution!"<<endl<<gMovesCount<<endl;
			for (int i=0;i<gMovesCount;i++)
			{
				cout<<gMoves[i]<<endl;
			}
		}
		cout<<10<<endl<<"OK!"<<endl;
	} 
	else
	{
		cout<<0<<endl<<"After all moves of the hero, he still did not get out"<<endl;
	}
}


int main (int argc,char **argv)
{
	ifstream fin(argv[1]);
	ifstream fout(argv[2]);
	ifstream fsol(argv[3]);

	int traps;

	fin>>gN>>gWappoRow>>gWappoCol>>gM1Row>>gM1Col>>gM2Row>>gM2Col>>traps;

	for (int i=0;i<traps;i++) 
	{
		int row;
		int col;
		fin>>row>>col;

		gTrap[row][col] = true;
	}

	for (int i=1;i<=gN;i++)
	{
		for (int j=1;j<=gN;j++)
		{
			fin>>gMatrix[i][j];
			if (i==1 && (!hasWall(i,j,WT_UP)))
			{
				gExitRow = i-1;
				gExitCol = j;
			}
			if (i==gN && (!hasWall(i,j,WT_DOWN)))
			{
				gExitRow = i+1;
				gExitCol = j;
			}
			if (j==1 && (!hasWall(i,j,WT_LEFT)))
			{
				gExitRow = i;
				gExitCol = j-1;
			}
			if (j==gN && (!hasWall(i,j,WT_RIGHT)))
			{
				gExitRow = i;
				gExitCol = j+1;
			}
		}
	}

	fin.close();

	int outMoves;

	fsol>>outMoves;

	fout>>gMovesCount;

   cout<<gMovesCount<<" "<<outMoves<<endl;
	if (gMovesCount>outMoves)
	{
		cout<<0<<endl<<"Moves count do not match!"<<endl;
	}
	else
	{
		for (int i=0;i<gMovesCount;i++)
		{
			fout>>gMoves[i];
		}
		if (gMovesCount==outMoves)
		{
			simulateGame(false);
		}
		else
		{
			simulateGame(true);
		}
		
	}

	return 0;
}
