/*
ID: pascala1
PROG: cownum
LANG: C++
Problem cownum: Numbering the Cows [Romanian Olympiad, 2001]

Farmer John wants to assign a (not necessarily distinct) integer
in the range 1..N (3 <= N <= 1,000) to each of his N cows for
identification purposes. Thus, he lined up all his cows in order
and came up with the following rule: the i-th cow in line must be
assigned an integer that is less than the integers assigned to cows
i+2 and i+3.

FJ wants to know the total number of possible assignments he might
make. If cow i+2 does not exist, then cow i does not have the
constraint that its ID number must be less than that of cow i+2.
Likewise if cow i+3 does not exist.

Since Farmer John is not too good at math and might be confused by
large numbers, output the answer modulo M (1 <= M <= 10,000,000).

PROBLEM NAME: cownum

INPUT FORMAT:

Two space-separated integers: N and M.

SAMPLE INPUT (file cownum.in):

3 10000

OUTPUT FORMAT:

A single line containing the number of assignments modulo M.

SAMPLE OUTPUT (file cownum.out):

9

OUTPUT DETAILS:

There are 9 valid assignments: (1,1,2), (1,1,3), (1,2,2), (1,2,3), (1,3,2),
(1,3,3), (2,1,3), (2,2,3), (2,3,3).
*/
//note all nums are < last two nums
//dp[maxval][pos] = answer for first pos positions with values <= maxvalue
// = dp[maxval-1][pos] (if no number is equal to maxval)
// + dp[maxval-1][pos-1] (if the last number is set to maxval and the pre-last is set to i=1..maxval-1 * )
// + dp[i-1][pos-2] (if the pre-last num is set to maxval and the last is set to i=1..maxval)
//* note that the case when both the last two numbers are set to maxval is included in the third addend
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <fstream>
#include <queue>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstdlib>
#include <ctime>


#define SZ size()
#define PB push_back
#define ALL(v) v.begin(),v.end()
#define MP make_pair
#define x first
#define y second

#define LL long long
#define UI unsigned int
#define ULL unsigned long long
#define PI pair<int,int>
#define PD pair<double,double>
#define PLL pair<LL,LL>
#define PULL pair<ULL,ULL>
#define VI vector<int>
#define VD vector<double>
#define VS vector<string>
#define SI set<int>
#define PQ priority_queue
#define VVI vector<vector<int> >
#define IT iterator

#define ABS(x) (((x)>0)?(x):(-(x)))
#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))
#define sign(a) ((a)>0)-((a)<0)
#define sqr(a) ((a)*(a))
#define Repi(n) for (int i=0; i<n; i++)
#define Repj(n) for (int j=0; j<n; j++)
#define Repk(n) for (int k=0; k<n; k++)

#define INF 2000000000
#define EPS 1e-6

#define Time ((double)clock()/CLOCKS_PER_SEC)
#define pause system("pause")

using namespace std;


#define MAXN 1024

int bans;
int ba[MAXN];
int N;
void bruteforce(int pos)
{
	if (pos==N)
	 {
	   bans++;
	   Repi(N) cout<<ba[i]<<" "; cout<<endl;
	   return;
	 }
	int from=1;
	if (pos>1) from=max(from,ba[pos-2]+1);
	if (pos>2) from=max(from,ba[pos-3]+1);
	for (ba[pos]=from;ba[pos]<=N;ba[pos]++)
	 bruteforce(pos+1);
}

int MOD=INF;
int dp[MAXN][MAXN];
int dp2[MAXN][MAXN];

int sumsolves(int maxmaxval,int pos);
int solve(int maxval,int pos);

int sumsolves(int maxmaxval,int pos)
{
	int &ret=dp2[maxmaxval][pos];
	if (ret>-1) return ret;
	if (!maxmaxval || !pos) return ret=0;
	return ret=(sumsolves(maxmaxval-1,pos)+solve(maxmaxval,pos))%MOD;
}

int solve(int maxval,int pos)
{
	if (maxval<1) return 0;
	int &ret=dp[maxval][pos];
	if (ret>-1) return ret;
	if (pos==1)
	 return ret=maxval%MOD;
	if (pos==2)
	 return ret=maxval*maxval%MOD;
	 
	//none = maxval
	ret=solve(maxval-1,pos);
	
    //last = maxval
    ret+=solve(maxval-1,pos-1);
    ret%=MOD;
    
    //prelast = maxval
	ret+=sumsolves(maxval-1,pos-2);
	ret%=MOD;
	
	return ret;
}


int main()
{
freopen("cownum.in","r",stdin);
freopen("cownum.out","w",stdout);
    cin>>N>>MOD;
    
//    N=6;	bans=0;	bruteforce(0);	cout<<bans<<"\n\n";
    
    memset(dp,-1,sizeof(dp));
    memset(dp2,-1,sizeof(dp2));
    cout<<solve(N,N)<<"\n";
    
  //  pause;
    return 0;
}
