/*
PROG:dice
ID:pascala1
LANG:C++
Problem dice: Different dice [Hal Burch, 1998]

In a different galaxy far far away there are different kinds of
dice, often with unusual numbers of sides -- anywhere from 2 to 16
sides. Each side can have from 1 to 32 spots. Sets of dice contain
anywhere from 1 through 32 dice. These dice roll just like Earth-
based dice, of course, and on any roll you might get any one of the
numbers on each die.

Anthropologists digging through old ruins in the galaxy far far
away have found sets of dice that have various different numbers
of spots.  Here's one set of two two-sided dice they found:

        * {2,3} and {3,4}

and here's another:

        * {1,2} and {4,5}

It's easy to see that the first set of dice yields sums of 5, 6,
and 7.  The second set also yields 5, 6, and 7.  Furthermore, the
probability of rolling a five is 1/4, rolling a six is 1/2, and
rolling a a seven is 1/4 for both sets of dice.

Given two sets of dice, your program must decide if they yield the
same sets of sums.  Additionally, it must also decide if the two
sets yield those sums with the same probabilities.

PROBLEM NAME: dice

INPUT FORMAT:

* Line 1: Two integers separated by a space, D1 and S1, representing
        the number of dice in the first set and the number of sides on
        each die.

* Lines 2..D1+1: S1 values that specify the spot values for a die. The
        values might or might not all be different.

* Line D1+2: Two integers separated by a space, D2 and S2. D2 is the
        number of dice in the second set, and S2 is the number of
        sides of each die.

* Lines D1+3..D1+D2+2: S2 values that specify the spot values for a
        die. The values might or might not all be different.

SAMPLE INPUT (file dice.in):

2 6
1 2 3 4 5 6
1 2 4 3 4 1
3 4
1 4 3 2
5 3 2 1
1 1 1 1

OUTPUT FORMAT:

* Line 1: Two characters separated by a space. First character is `Y'
        if both sets of dice yield the same set of sums or `N'
        otherwise. The second character is `Y' if both sets of dice
        yield the same set of possible sums with the same set of
        probabilities of achieving those sums or `N' otherwise.

* Line 2: Two space-separated integers. The first integer indicates
        the number of possible sums the first set of dice could yield.
        The second integer indicates the number of different sums the
        second set of dice could yield.

SAMPLE OUTPUT (file dice.out):

N N
9 8
*/
#include <cstdio>
#include <cstring>
#include <cmath>

#define Repi(n) for (int i=0;i<n;i++)
#define Repj(n) for (int j=0;j<n;j++)
#define Repk(n) for (int k=0;k<n;k++)

int S1,D1,S2,D2,d1[64][64],d2[64][64];
double sp1[64*64],sp2[64*64];

void solve(int S,int D,int d[][64],double sprob[])
{
	  int q[2][64*64],qc[2]; qc[0]=1; q[0][0]=0;
	  double dp[2][64*64]; memset(dp,0,sizeof(dp)); dp[0][0]=1;
	  int at=0;
	  double pr=1./(double)D;
	  Repi(S)
	   {
			memset(dp[at^1],0,sizeof(dp[at^1]));
			qc[at^1]=0;
		//	printf(" at die %d\n",i);
			Repj(qc[at])
			 {
					int cs=q[at][j];
					Repk(D)
					 {
					    int ns=cs+d[i][k];
					    if (!dp[at^1][ns]) q[at^1][qc[at^1]++]=ns;
					    dp[at^1][ns]+=dp[at][cs]*pr;
					 }
			 }
			at^=1;
		//	printf(" after die %d : \n",i); Repj(qc[at]) printf("               %d - %lf\n",q[at][j],dp[at][q[at][j]]);
	   }
	 memcpy(sprob,dp[at],sizeof(dp[at]));
}

int main()
{
	freopen("dice.in","r",stdin); freopen("dice.out","w",stdout);
	
	scanf("%d%d",&S1,&D1);
	fprintf(stderr,"%d %d\n",S1,D1);
	Repi(S1)
	 {
			Repj(D1)
		     scanf("%d",&d1[i][j]),fprintf(stderr,"%d ",d1[i][j]);;
			fprintf(stderr,"\n");
	 }

	scanf("%d%d",&S2,&D2);
	fprintf(stderr,"%d %d\n",S2,D2);
	Repi(S2)
	 {
			Repj(D2)
			 scanf("%d",&d2[i][j]),fprintf(stderr,"%d ",d2[i][j]);;
			fprintf(stderr,"\n");
	 }
	 
	solve(S1,D1,d1,sp1);
//	printf("\n\n\n\nddsffffffffffffffffffffffffffffffffffffffffffffffffffffgdsgdgsdgdsf\n\n\n");
	solve(S2,D2,d2,sp2);
	
	char a1='Y',a2='Y';
	int c1=0,c2=0;
	Repi(64*64)
	 {
	   if ( (sp1[i]>0) != (sp2[i]>0) ) a1='N';
	   if ( sp1[i] != sp2[i] ) a2='N';
	   if (sp1[i]>0) c1++; if (sp2[i]>0) c2++;
	 }
	printf("%c %c\n%d %d\n",a1,a2,c1,c2);
	
	return 0;
}
