/*
PROG:cowq
LANG:C++
ID:pascala1
Problem cowq: Cow Queuing [Marius Andrei (Romanian National Informatics Olympiad 2005), 2006]

Farmer John heard about a new way to re-order his N (2 <= N <= 100)
cows conveniently numbered 1..N on their way to the barn for dinner.
He built a special corral arrangement that looks like this:

              ______________     ________________
                 <-- Exit . \   / . <-- Entrance
               ___________ . \ / . ______________
                          \ . V . /
                           \ . . /
                            | . |
                            | . |
             Holding Pen -->| . |
                            :   :  <-- lots of space not shown here
                            | . |
                            -----

All the cows line up to the right of the entrance. Each cow will
eventually exit to the left, but must first go into the holding
pen. The holding pen is just wide enough for one cow, and thus acts
as a sort-of stack: last-cow-in=>first-cow-out. At any reasonable
interval, FJ can direct the cows in one of two ways:

* Direct the cow at the front of the entrance to go into the holding pen.

* Direct the cow that has most recently entered the holding area to the exit.

FJ realizes that by using this corral he can reorder the cows in a
large number of ways. FJ chooses a sequence of instructions uniformly
from among all possible valid instruction sequences. With three
cows in line (sequentially numbered 1, 2, 3) he can effect five
final orderings using five instruction sequences:

                Instructions                              Final Ordering
#1: 1 to exit, 2 to exit, 3 to exit                           1, 2, 3
#2: 1 to exit, 2 to queue, 3 to exit, 2 to exit               1, 3, 2
#3: 1 to queue, 2 to exit, 1 to exit, 3 to exit               2, 1, 3
#4: 1 to queue, 2 to exit, 3 to exit, 1 to exit               2, 3, 1
#5: 1 to queue, 2 to queue, 3 to exit, 2 to exit, 1 to exit   3, 2, 1

Bessie is watching FJ implement this scheme. She always wishes to
eat ahead of her rival, Joanne. Given Bessie's initial position (B)
and Joanne's initial position (J) calculate the fraction of valid
instruction sequences which will result in Bessie ending ahead of
Joanne.

Express this fraction as a decimal between 0 and 1. Your output
should start with a 0 or a 1, have a decimal point, and then have
zero or more decimal digits. A special grader will accept all answers
within 0.0001 of the official answer.

In the case above, if Bessie is cow #1 and Joanne is cow #3, then
Bessie will eat first 60% of the time, with sequences #1, #2, and #3.

PROBLEM NAME: cowq

INPUT FORMAT:

* Line 1: Three space-separated integers: N, B, and J

SAMPLE INPUT (file cowq.in):

3 1 3

OUTPUT FORMAT:

* Line 1: A decimal number expressed as a floating point number that
        is the fraction of the time Bessie will eat before Joanne if
        FJ directs cows randomly.

SAMPLE OUTPUT (file cowq.out):

0.600000
*/
// bfs dp
// bug1: too much mem for usaco...i kato prai6 rotating queue, aide rotate-vai i start-a osven end-a...i kato prai6 rotating queue, ai pravi go na si4ki mesta kudeto puska6 bfs...
// bug2: OVERFLOW!!!! taka i taka deli6! - use double 4e da pazi6 samo most significant digits!
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <ctime>
#include <cmath>
#include <vector>
#include <bitset>
#include <set>
#include <queue>
#include <map>
#include <utility>
#include <algorithm>
#include <functional>
#include <numeric>

using namespace std;

#define Repi(n) for (int i=0;i<n;i++)
#define Repj(n) for (int j=0;j<n;j++)
#define Repk(n) for (int k=0;k<n;k++)

#define PI pair<int,int>
#define PB push_back
#define VI vector<int>
#define MP make_pair
#define x first
#define y second
#define PD pair<double,double>
#define LL long long
#define SZ size()
#define PQ priority_queue
#define sign(x) (((x)>0)-((x)<0)))

struct three{int q,s,a; three(){}; three(int qq,int ss,int aa):q(qq),s(ss),a(aa){};};

int N,B,L;
double dp[102][102][102]; //cnt in queue, cnt in stack, cnt above bessie (if she's in stack, otherwise = N)
const int MAXQ=102*102*10;
three qu[MAXQ]; int st,en;
char indeg[102][102][102];
bool dontfuckbessie;

void getindeg()
{
	memset(indeg,0,sizeof(indeg));
	qu[0]=three(N,0,N);
	int nq,ns,na;
	for (st=0,en=1;st<en;st++)
	 {
			if (st==MAXQ) st=0;
			int q=qu[st].q,s=qu[st].s,a=qu[st].a;
			if (q && !(dontfuckbessie && a<N && q==L))
			 {
					nq=q-1,ns=s+1,na=(a==N)?(q==B)?0:N:a+1;
					if (!indeg[nq][ns][na])
					 qu[en++]=three(nq,ns,na);
					if (en==MAXQ) en=0;
					indeg[nq][ns][na]++;
			 }
			if (s)
			 {
					nq=q,ns=s-1,na=(a!=N)?(a==0)?N:a-1:N;
					if (!indeg[nq][ns][na])
					 qu[en++]=three(nq,ns,na);
					if (en==MAXQ) en=0;
					indeg[nq][ns][na]++;
			 } 
	 }
}

int addstate(int q,int s,int a,int nq,int ns,int na)
{
	dp[nq][ns][na]+=dp[q][s][a];
	indeg[nq][ns][na]--;
	//cout<<"   to "<<nq<<" "<<ns<<" "<<na<<" += "<<dp[q][s][a]<<" = "<<dp[nq][ns][na]<<" ; "<<indeg[nq][ns][na]<<endl;
	if (!indeg[nq][ns][na])
	 {
	  // cout<<"                  adding state...\n";
	   qu[en++]=three(nq,ns,na);
	   if (en==MAXQ) en=0;
	 }
}

double solve()
{
	getindeg();
	
	memset(dp,0,sizeof(dp)); dp[N][0][N]=1;
	qu[0]=three(N,0,N); dp[N][0][N]=1;
	int nq,ns,na;
	for (st=0,en=1;st!=en;st++)
	 {
			if (st==MAXQ) st=0;
			int q=qu[st].q,s=qu[st].s,a=qu[st].a;
		//	cout<<" in "<<q<<" "<<s<<" "<<a<<endl;
			if (q && !(dontfuckbessie && a<N && q==L))
			 {
					nq=q-1,ns=s+1,na=(a==N)?(q==B)?0:N:a+1;
					addstate(q,s,a,nq,ns,na);
			 }
			if (s)
			 {
					nq=q,ns=s-1,na=(a!=N)?(a==0)?N:a-1:N;
					addstate(q,s,a,nq,ns,na);
			 } 
	 }
	return dp[0][0][N];
}

int main()
{
	freopen("cowq.in","r",stdin); freopen("cowq.out","w",stdout);
	scanf("%d%d%d",&N,&B,&L);
	B=N-B+1; L=N-L+1;
	bool swapped=0;
	if (B<L) swap(B,L),swapped=1;
	//cout<<B<<" "<<L<<" , "<<swapped<<endl;
	
	dontfuckbessie=0;
	double all=solve();
	dontfuckbessie=1;
	double bessie=solve();
	
	double ans=bessie/all;
	if (swapped) ans=1.-ans;
	
	printf("%.6lf\n",ans);	
	return 0;
}

