/*
PROG:storage
LANG:C++
ID:pascala1
Problem storage: Haybale Storage [Brian Dean, 2004]

Farmer John would like to store haybales in one of his old
two-dimensional barns whose integer width is B (1 <= B <= 250) units
and whose integer height H_i (at x=i) is also in the range (1 <=
H_i <= 250). The trouble with this particular barn is that it has
a very sloped roof. A cross section of the barn looks something
like this (in particular, the roof always slopes upward then back
downward):

           xxxxxxxxxx 
       xxxx          xxxxxx 
    xxx                    xx
  xx                        x
 x          (inside)        x 
x                           x  

Each of FJ's N (1 <= N <= 250) bales of hay has height H_i (1..250)
and width W_i (1..250) . These fragile hay bales may neither be
rotated nor stacked on top of each other, so one packing of a set
of hay bales might look something like this:

           xxxxxxxxxx 
       xxxx555       xxxxxx 
    xxx444 555             xx
  xx22 444 555          7777x
 x  223444 555 6666666  7777x 
x111223444 555 6666666  7777x  

It's a pity one can't stack more bales on top of the 6's, but that's
the rule.

Given the width and height of each bale of hay and a description
of the sloped roof, determine the maximum amount of hay (in terms
of total cross sectional area) that can be packed into the barn.

PROBLEM NAME: storage

INPUT FORMAT:

* Line 1: Two space-separated integers: N and B

* Lines 2..N+1: Line i+1 describes bale i with two space-separated
        integers: W_i and H_i

* Lines N+2..N+1+B: Line N+i+1 contains a single integer: H_i

SAMPLE INPUT (file storage.in):

4 6
4 1
2 1
2 2
1 3
1
2
2
3
2
1

INPUT DETAILS:

The barn looks like this:
    x 
  xx x 
 x    x 
x      x

OUTPUT FORMAT:

* Line 1: One integer, the maximum obtainable cross-sectional area.

SAMPLE OUTPUT (file storage.out):

9

OUTPUT DETAILS:

Bales 2, 3 and 4 can be packed like this:
   xxx
 xxx4xx
xx334 xx
x 33422x
*/
//recursive to iterative (mem cut)
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <ctime>
#include <cmath>
#include <vector>
#include <bitset>
#include <set>
#include <queue>
#include <map>
#include <utility>
#include <algorithm>
#include <functional>
#include <numeric>

using namespace std;

#define Repi(n) for (int i=0;i<n;i++)
#define Repj(n) for (int j=0;j<n;j++)
#define Repk(n) for (int k=0;k<n;k++)

#define PI pair<int,int>
#define PB push_back
#define VI vector<int>
#define MP make_pair
#define x first
#define y second
#define PD pair<double,double>
#define LL long long
#define SZ size()
#define PQ priority_queue
#define sign(x) (((x)>0)-((x)<0)))

#define MAXN 252

PI p[MAXN];
int h[MAXN],N,B;
int ***dp;
int minheight[MAXN][MAXN];

int solve(int st,int en,int at)
{
	if (st>en) return 0;
	int &ret=dp[st][en][at];
	if (ret>-1) return ret;
	ret=0;
	if (at==N) return ret;
	
	ret=max(ret,solve(st,en,at+1));
	ret=max(ret,solve(st+1,en,at));
	ret=max(ret,solve(st,en-1,at));
	if (en-st+1>=p[at].y && minheight[st][en]>=p[at].x)
	 {
			ret=max(ret,solve(st+p[at].y,en,at+1)+p[at].x*p[at].y);
			ret=max(ret,solve(st,en-p[at].y,at+1)+p[at].x*p[at].y);
	 }
	
	//cout<<" "<<st<<" "<<en<<" "<<at<<" = "<<ret<<endl;
	return ret;
}

int solveit()
{
	int dp1[2][MAXN][MAXN]; //at moved as first parameter; interval coordinates ++ (corner cases st>en)
	memset(dp1[0],0,sizeof(dp[0]));
	int Y=0;
	for (int at=N;at;at--)
	 {
			memset(dp1[Y^1],0,sizeof(dp1[Y^1]));
			for (int st=B;st;st--)
			 for (int en=st;en<=B;en++)
			  {
				int &ret=dp1[Y^1][st][en];
			    ret=max(ret,dp1[Y][st][en]);
			    ret=max(ret,dp1[Y^1][st+1][en]);
			    ret=max(ret,dp1[Y^1][st][en-1]);
			    if (en-st+1>=p[at-1].y && minheight[st-1][en-1]>=p[at-1].x)
			     ret=max(ret,dp1[Y][st+p[at-1].y][en]+p[at-1].x*p[at-1].y),
				 ret=max(ret,dp1[Y][st][en-p[at-1].y]+p[at-1].x*p[at-1].y);
			  }
			Y^=1;
	 }
	return dp1[Y][1][B];
}

int main()
{
	freopen("storage.in","r",stdin); freopen("storage.out","w",stdout);
	scanf("%d%d",&N,&B);
	Repi(N)
	 scanf("%d%d",&p[i].y,&p[i].x);
	sort(p,p+N);
    Repi(B)
     scanf("%d",h+i);
//	Repi(N) cout<<i<<" : "<<p[i].y<<" wide x "<<p[i].x<<" tall\n";

/*
    dp=new int**[B+1];
    Repi(B+1) 
     {
			dp[i]=new int*[B+1];
			Repj(B+1)
			 {
			   dp[i][j]=new int[N+1];
			   memset(dp[i][j],-1,sizeof(int)*(N+1));
			 }
	 }
	 */
	for (int st=0;st<B;st++)
	 for (int en=st;en<B;en++)
	  {
	    minheight[st][en]=1000000000;
	    for (int i=st;i<=en;i++)
	     minheight[st][en]=min(minheight[st][en],h[i]);
	  } 
	 
//	memset(dp,-1,sizeof(dp));
//	printf("%d\n",solve(0,B-1,0));
	printf("%d\n",solveit());
	return 0;
}

