/*
PROG:palpath
LANG:C++
ID:pascala1

Problem palpath: Palindromic Tax Paths [Alex Schwendner, 2002]

The cows are bored. To amuse themselves, they have started searching
for palindromes in Farmer John's tax forms. Palindromes are numbers
that read the same forwards and backwards, e.g., 1234321 (but not
1231). A palindrome can be as short as 1 digit in length.

Each tax form is an N x N (2 <= N <= 20) table of random single
digits (0..9). The cows wish to find paths on the grid such that
the sequence of digits the path traces is a palindrome. For example,
in the table below, the path shown traces out the palindrome 122221:

1-2 0 0
   \
3 8 2=2
   /
3 1 8 9

3 4 5 4

Some of the other palindromes that exist in the above example are
000, 121, 131, 1331, 13331, 2002 (many different ways), 318813,
454, and 8338. The paths may loop back upon themselves (as is the
case with, e.g., 121, 122221, and 000). However, the paths may not
use the same table entry more than once sequentially (i.e., 9889,
95559, and 9999999 are not valid palindromes). The length of a
palindromic path is the number of digits it uses (12221 uses 5
digits).

The cows would like to know how many palindromic paths of a given
length L (1 <= L <= 18) exist. Write a program to determine the
number of palindromic paths of length L for a given table of digits.
If a path and its inverse are distinct, count them as two paths.
That is, if traversing a path backwards is not the same as traversing
it forwards, count it as two paths. In the example table above,
count 122221 as two paths because it can be traversed either starting
on the 1 in the first row and ending on the 1 in the third row, or
starting on the 1 in the third row and ending on the 1 in the first
row. On the other hand, count `949' only once, as it can only be
traversed starting at 9, going to 4, and returning to the single
9.

PROBLEM NAME: palpath

INPUT FORMAT:

* Line 1: Two space-separated integers: N and L

* Lines 2..N+1: The tax form: N lines of N space-separated digits

SAMPLE INPUT (file palpath.in):

3 3
1 2 3
1 2 3
1 2 3

OUTPUT FORMAT:

* Line 1: A single integer: the total number of palindromic paths. 
        All answers for this problem's test data fit into signed 32
        bit entities.

SAMPLE OUTPUT (file palpath.out):

86
*/
// dp[len][x1][y1][x2][y2] = number of palindromes with ends x1,y1 and x2,y2 and length len (solution depicts dp as getting the opposite ends closer together (eventually uniting them in the final iteration))
// could also be done the other way round - start building palindrome from the middle
// !!!!! the only working optimization - for each [x1][y1][dig] - coordinates of all neighbours of cell x1,y1 that have value dig
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <ctime>
#include <cmath>
#include <vector>
#include <bitset>
#include <set>
#include <queue>
#include <map>
#include <utility>
#include <algorithm>
#include <functional>
#include <numeric>

using namespace std;

#define Repi(n) for (int i=0;i<n;i++)
#define Repj(n) for (int j=0;j<n;j++)
#define Repk(n) for (int k=0;k<n;k++)

#define PI pair<int,int>
#define F(i,n) for (int i=0;i<n;i++)
#define PB push_back
#define VI vector<int>
#define MP make_pair
#define x first
#define y second
#define PD pair<double,double>
#define LL long long
#define SZ size()
#define PQ priority_queue
#define sign(x) (((x)>0)-((x)<0)))

#define MAXN 21
int dp[2][MAXN][MAXN][MAXN][MAXN];
int mv[8][2]={
	{0,1},
	{1,0},
	{0,-1},
	{-1,0},
	{1,1},
	{1,-1},
	{-1,1},
	{-1,-1}
	};
int N,L,a[MAXN][MAXN];

#define haha Repi(N) Repj(N) F(k,N) F(l,N)

vector<PI> adj[MAXN][MAXN][10];

int main()
{
	freopen("palpath.in","r",stdin); freopen("palpath.out","w",stdout);
	scanf("%d%d",&N,&L);
	Repi(N)
	 Repj(N)
	  scanf("%d",&a[i][j]);

	Repi(N)
	 Repj(N)
	  Repk(8)
	   {
			int ni=i+mv[k][0],nj=j+mv[k][1];
			if (ni<0 || ni>=N || nj<0 || nj>=N) continue;
			adj[i][j][a[ni][nj]].PB(MP(ni,nj));
	   }

	haha
	 dp[0][i][j][k][l]=(a[i][j]==a[k][l]);
	int at=0;
	
	for (int len=2;len<=L/2+L%2;len++)
	 {
			memset(dp[at^1],0,sizeof(dp[at^1]));
			haha
			 {
					if (!dp[i][j][k][l]) continue;
					F(m,8)
					 {
							int ni=i+mv[m][0],nj=j+mv[m][1];
							if (ni<0 || ni>=N || nj<0 || nj>=N) continue;
							int d=a[ni][nj];
							F(t,adj[k][l][d].SZ)
							 {
									int nk=adj[k][l][d][t].x,nl=adj[k][l][d][t].y;
									dp[at^1][ni][nj][nk][nl]+=dp[at][i][j][k][l];
							 }
					 }
			 }
			at^=1;
	 }
	 
	 
	int ans=0;
	if (L%2)      //  pals with len L/2+1 have met in the middle (the middle element is common)
	 {
			Repi(N)
			 Repj(N)
			  ans+=dp[at][i][j][i][j];
	 }
	else          //  the two ends only touch - they do not overlap
	 {
			Repi(N)
			 Repj(N)
					F(m,8)
					 {
					   int ni=i+mv[m][0],nj=j+mv[m][1];
					   if (ni<0 || ni>=N || nj<0 || nj>=N) continue;
					   ans+=dp[at][i][j][ni][nj];
					 }
	 }
	printf("%d\n",ans);
	return 0;
}

