/*
ID: pascala1
PROG: traffic
LANG: C++

Problem traffic: Traffic Lights [J. Kuipers, 2002]

The cows are going downtown! Just like everyone else, they want to
optimize their driving time.

They have noted that when driving on a straight road with traffic
lights, the best strategy to get to their destination as quickly
as possible is not necessarily to drive as fast as possible to the
next traffic light, brake if it's red, wait for a green light,
accelerate, and then drive on. It is often better to approach a
traffic light more slowly in order to have some speed when the light
turns green.

The cows have observed the traffic lights for a very long time.
They know that each traffic light behaves in the following way:
  * it is green for a certain amount of time Tg_i,
  * then it is red for an amount of time Tr_i,
  * then green again,
  * and so on.

Given
  * the integer length of the road L (1 <= L <= 100)
  * the number of traffic lights N (0 <= N <= L+1) along with
    information about each light:
     * P_i: the unique position (0 <= P_i <= L)
     * Tg_i (1 <= Tg_i <= 10)
     * Tr_i (1 <= Tr_i <= 10)
     * c_i: color at t=0 (R or G)
     * Tc_i (the integer amount of time since the light last changed)

write a program to determine the minimal amount of time needed to
get to the end of the road. Note that at each discrete time (starting
at t=0), a car may either change its speed (expressed in positional
units per time unit) by one or keep it constant. The speed is always
0 or positive, of course. No driving backwards!

The car starts at position zero has has speed zero. The car must
complete its trip at position L, also with speed zero. The car must
stop at all red lights -- be sure its speed is 0 at the red light's
position if it encounters a red light. The car may move when the
light changes from red to green, but not when it changes from green
to red.

PROBLEM NAME: traffic

INPUT FORMAT:

* Line 1: Two space-separated integers: L and N

* Lines 2..N+1: Line i+1 describes traffic light i with five
        space-separated entities (all are integers except #4 which is
        a single character 'R' or 'G'): P_i, Tg_i, Tr_i, c_i, and Tc_i

SAMPLE INPUT (file traffic.in):

4 1
1 10 10 R 0

OUTPUT FORMAT:

A single line with a single integer that is the minimal time needed.

SAMPLE OUTPUT (file traffic.out):

12
*/
// note: speed <= 10 ; time <= 1000
// dp [ pos ][ speed ][ time elapsed ] = minimum time to go to end
// precomputed red[ time ][ pos ] = number of red lights in [0..pos]
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <fstream>
#include <queue>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstdlib>
#include <ctime>


#define SZ size()
#define PB push_back
#define ALL(v) v.begin(),v.end()
#define MP make_pair
#define x first
#define y second

#define LL long long
#define UI unsigned int
#define ULL unsigned long long
#define PI pair<int,int>
#define PD pair<double,double>
#define PLL pair<LL,LL>
#define PULL pair<ULL,ULL>
#define VI vector<int>
#define VD vector<double>
#define VS vector<string>
#define SI set<int>
#define PQ priority_queue
#define VVI vector<vector<int> >
#define IT iterator

#define sign(a) ((a)>0)-((a)<0)
#define sqr(a) ((a)*(a))
#define Repi(n) for (int i=0; i<n; i++)
#define Repj(n) for (int j=0; j<n; j++)
#define Repk(n) for (int k=0; k<n; k++)

#define Pi 3.1415926535897932384626433832795028841971693993751
#define INF 100000000
#define EPS 1e-6

#define pause system("pause")

using namespace std;
#define MAXT 1024

int N,L;
int dp[102][12][MAXT];
int next[102][12][MAXT];
struct light{ int pos,rt,gt,tc; char c; 
char col(int tim)
{ tim=(tim+tc)%(gt+rt);
  if ((c=='R' && tim<rt) || (c=='G' && tim>=gt)) return 'R';
  return 'G';
 };
};
light a[102];
int atpos[102];
int red[MAXT][102];

bool canpass(int ps,int pe,int tm)
{
	if (!pe) return 1;
	int rr=(ps)?red[tm][pe-1]-red[tm][ps-1]:red[tm][pe-1];
	return rr==0;
}

int solve(int pos,int spd,int tm)
{
	if (spd<0 || spd>11 || pos>L || tm>=MAXT) return INF;
	int &ret=dp[pos][spd][tm];
	if (ret>-1) return ret;
	ret=INF;
	if (pos==L) if (spd>1) return ret; else return ret=0;
	
    //cout<<" at "<<pos<<" "<<spd<<" "<<tm<<endl;
    int l=atpos[pos],tmp;
    if (l==-1 || a[l].col(tm)=='G')
     {
		//	cout<<"      can pass!!!"<<" : "<<l<<" "<<a[l].col(tm)<<"\n";
			if (canpass(pos,pos+spd,tm))
			 { tmp=solve(pos+spd,spd,tm+1) + 1;  if (tmp<ret) next[pos][spd][tm]=0,ret=tmp; };
			if (canpass(pos,pos+spd+1,tm)) 
			 { tmp=solve(pos+spd+1,spd+1,tm+1) + 1;  if (tmp<ret) next[pos][spd][tm]=1,ret=tmp; };
			if (spd && canpass(pos,pos+spd-1,tm))
			 { tmp=solve(pos+spd-1,spd-1,tm+1) + 1;  if (tmp<ret) next[pos][spd][tm]=-1,ret=tmp; };
	 }
	else
	  {
		//	cout<<"            red :(\n";
			if (spd>1) return ret=INF;
			next[pos][spd][tm]=0;
			ret=solve(pos,0,tm+1)+1;
	  }
//	cout<<"==== "<<pos<<" "<<spd<<" "<<tm<<" = "<<ret<<endl;
	return ret;
};


int main()
{
    int i,j,k,l;
    freopen("traffic.in","r",stdin);
    freopen("traffic.out","w",stdout);

    memset(atpos,-1,sizeof(atpos));
    scanf("%d%d",&L,&N);
    Repi(N)
     scanf("%d%d%d %c %d",&a[i].pos,&a[i].gt,&a[i].rt,&a[i].c,&a[i].tc),atpos[a[i].pos]=i;

/*
    Repi(N)
     {
	    cout<<" light "<<i<<" : "<<a[i].pos<<" "<<a[i].gt<<" "<<a[i].rt<<" "<<a[i].c<<" "<<a[i].tc<<endl;
	    Repj(60)
	     cout<<"   time "<<j<<" : "<<a[i].col(j)<<endl;
	 }*/
	for (int i=0;i<MAXT;i++)
	 {
	   red[i][0]=(atpos[0]==-1)?0:(a[atpos[0]].col(i)=='R');
	   for (int j=1;j<=L;j++)
	    red[i][j]=red[i][j-1] + ((atpos[j]==-1)?0:(a[atpos[j]].col(i)=='R'));
	   //cout<<" lights tm "<<i<<" : "; Repj(L+1) cout<<red[i][j]; cout<<endl;
	 }
	
	memset(dp,-1,sizeof(dp));
	memset(next,127,sizeof(next));
	printf("%d\n",solve(0,0,0));
    
    /*
    int pos=0,spd=0,tm=0;
    while (pos<L)
     {
			if (next[pos][spd][tm]>INF) { cout<<"   route fucked up!!! \n"; break; }
			spd+=next[pos][spd][tm];
			pos+=spd; tm++;
			cout<<"  time "<<tm<<" : at "<<pos<<" with "<<spd<<endl;
	 }*/
    
    return 0;
}
