/*
ID: pascala1
PROG: twofour
LANG: C++

Problem twofour: TwoFour [Romanian Olympiad, via Stroe, 2002]

Bessie has a new two-person game named TwoFour. To play it, she has
N (3 <= N <= 30) piles, each with a number of balls (0 <= nballs
<= 4). The total number of balls is 2*N.

The players alternate taking turns in the game; player 1 makes the
first move. Each turn consists of a single valid move.  A valid
move consists of the following two actions:

  * First, the player chooses two different piles.

  * Second, she takes a single ball from one pile and moves it to
    the other pile. She can do this only if the number of balls in
    the second pile (including the new ball) is not greater than
    the number of balls remaining in the first pile after the ball
    is removed.

The game ends when no more moves can be made. In fact, at the end of
the game, every pile will contain exactly two balls.

The winner of the game is the player who 'owns' most piles at the end
of the game.  Ties are possible.  A player 'owns' a pile if the pile
has two balls and this size resulted from the particular player's
most recent move to or from that pile.  Consider these examples:

    * If a player moves a ball from a pile with four balls to a
      pile with one ball, then she owns the second pile (with two
      balls).

    * If a player moves a ball from a pile with three balls to a
      pile with no balls, then she owns the first pile, now that
      it has two balls.

    * If a player moves a ball from a pile with three balls to a pile
      with one ball, then she owns both piles (both with two balls).

Ownership can change. Consider a pile of two balls owned by one
player. If the other player chooses a pile with four balls and moves
a ball to the pile with two, then neither pile is owned by anyone.

If, at the beginning of the game, some piles have two balls, then
the piles are equally distributed among the two players with any
extra pile being owned by player two.

Your program must decide, for an initial game state, who will be
the winner or if the game ends in a tie.  Of course, the decision
is based on optimal play by both players. Your program will be
presented with G (1 <= G <= 1000) game states to 'solve'.

Use no more than 1.000 Megabytes of memory for this problem.

PROBLEM NAME: twofour

INPUT FORMAT:

* Line 1: Two space-separated integers: N and G.

* Lines 2..G+1: Each line contains N space-separated integers
        describing a game.  The first integer is the number of balls
        in pile 1, the second integer is the number of balls in pile
        2, and so on.  Line 2 describes game 1, line 3 describes game
        2, and so on.  Your program should compute the winner for each
        game.

SAMPLE INPUT (file twofour.in):

5 4
0 3 4 1 2
2 2 2 2 2
1 1 2 2 4
4 3 2 1 0

OUTPUT FORMAT:

* Lines 1..G: The output contains the outcome of each game.  Line 1
        gives the outcome of game 1, and so on.  The outcome is a
        single integer:  1 if the first player wins, 2 if the second
        player wins, and 0 if the game is a tie.

SAMPLE OUTPUT (file twofour.out):

1
2
1
1
*/
// obviously is acyclic
// dp[ cnt_piles_size_0 ][ cnt_piles_size_1 ]..[ cnt_piles_size_4 ][ points_of_1 ][ points_of_2 ]
// compress - only need cnt for 4 sizes (cnt of the 5th = N - sum of others) ; since cnt0<=15, cnt1<=20, cnt2<=30, cnt3<=20, cnt4<=15 it's best to eliminate cnt of 2
// score of player 2 = cnt of size 2 - score of player 1
// we have 4 possible states for dp - unused, tie, 1 wins, 2 wins => final parameter (score of player 1) can be split in 2 ints (each storing 30 bits)
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <fstream>
#include <queue>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstdlib>
#include <ctime>


#define SZ size()
#define PB push_back
#define ALL(v) v.begin(),v.end()
#define MP make_pair
#define x first
#define y second

#define LL long long
#define UI unsigned int
#define ULL unsigned long long
#define PI pair<int,int>
#define PD pair<double,double>
#define PLL pair<LL,LL>
#define PULL pair<ULL,ULL>
#define VI vector<int>
#define VD vector<double>
#define VS vector<string>
#define SI set<int>
#define PQ priority_queue
#define VVI vector<vector<int> >
#define IT iterator

#define sign(a) ((a)>0)-((a)<0)
#define sqr(a) ((a)*(a))
#define Repi(n) for (int i=0; i<n; i++)
#define Repj(n) for (int j=0; j<n; j++)
#define Repk(n) for (int k=0; k<n; k++)

#define Pi 3.1415926535897932384626433832795028841971693993751
#define INF 2000000000
#define EPS 1e-6

#define pause system("pause")

using namespace std;

int n,m,G,N;
const int SIZE=16*21*21*16;

int d1[SIZE],d2[SIZE];   // 0 0 - unused; 0 1 - tie ; 1 0 - 1 wins ; 1 1 - 2 wins
int toans[2][2]={ {-1,0}, {1,2} };
int toshit[3][2]={ {0,1}, {1,0}, {1,1} };
void declare(int code,int pts1,int state)
{
	d1[code]|=toshit[state][0]<<pts1;
	d2[code]|=toshit[state][1]<<pts1;
}

int encode(int a[])
{
	int res=a[0]; res*=21,res+=a[1],res*=21,res+=a[3],res*=16,res+=a[4];
	return res;
}

void decode(int res,int a[])
{
	a[4]=res%16; res/=16; a[3]=res%21; res/=21; a[1]=res%21; res/=21; a[0]=res%16;
	a[2]=N-a[0]-a[1]-a[3]-a[4];
}

int solve(int a[],int pts1)
{
	int x=encode(a);
	bool a1=(d1[x]>>pts1)&1,a2=(d2[x]>>pts1)&1;
 //   cout<<" in "; Repi(5) cout<<a[i]<<" "; cout<<"("<<pts1<<" "<<a[2]-pts1<<")"<<"  -  "<<a1<<" "<<a2<<endl;
    if (toans[a1][a2]>-1) return toans[a1][a2];
    int pts2=a[2]-pts1;

	int state,na[5];
	memcpy(na,a,sizeof(na));  //careful! need to utilize new array or else the same one will be used in all recursions
	
    if (a[2]==N)
		state=(pts1==pts2)?0:1+(pts2>pts1);
	else
     {
      state=2;
      for (int i=4;i>1;i--)
       if (a[i])
        for (int j=0;j<i-1;j++)
         if (a[j])
          {
              int npts1=pts1,npts2=pts2;
		  	  na[i]--,na[i-1]++,na[j]--,na[j+1]++;
		      if (i==2 || j==2) if (npts2) npts2--; else npts1--;
	  		  npts1+=(i==3)+(j==1);
			  int nstate=solve(na,npts2);
			  
	  	//	  cout<<"  from "; Repk(5) cout<<a[k]<<" "; cout<<"("<<pts1<<" "<<pts2<<") ["<<state<<"]  to  ";
		//	  Repk(5) cout<<na[k]<<" "; cout<<"("<<npts2<<" "<<npts1<<") ["<<nstate<<"]"<<endl;
			  
			  if (nstate==0 && state==2) state=0;
			  if (nstate==2) { state=1; goto fuckbreak; }
			  na[i]++,na[i-1]--,na[j]++,na[j+1]--;
	      }
	 }
	 
	fuckbreak:
	//cout<<" DECLARE-- "; Repi(5) cout<<a[i]<<" "; cout<<"("<<pts1<<" "<<pts2<<") as "<<state<<endl;
	declare(x,pts1,state);
	return state;
}

int main()
{
    int i,j,k,l;
    freopen("twofour.in","r",stdin);
    freopen("twofour.out","w",stdout);
    scanf("%d%d",&N,&G);
    int a[5],r[N+2];
    Repi(G)
     {
			memset(a,0,sizeof(a));
			Repi(N)
			 scanf("%d",r+i);
			Repi(N) a[r[i]]++;
			
			printf("%d\n",solve(a,a[2]/2));
     }
    
    return 0;
}
