/*
PROG:lilypad
LANG:C++
ID:pascala1

Problem lilypad: Lilypad Pond [Richard Ho, 2006]

FJ has installed a beautiful pond for his cows' aesthetic enjoyment
and exercise.

The rectangular pond has been partitioned into square cells of M
rows and N columns (1 <= M <= 30; 1 <= N <= 30). Some of the cells
have astonishingly sturdy lilypads; others have rocks; the remainder
contain just beautiful, cool, blue water.

Bessie is practicing her ballet moves by jumping from one lilypad
to another and is currently located at one of the lilypads. She
wants to travel to another lilypad in the pond by jumping from one
lilypad to another.

Surprising only to the uninitiated, Bessie's jumps between lilypads
always appear as a chess-knight's move: one move in one direction
and then two more in the orthogonal direction (or perhaps two in
one direction and then one in the orthogonal direction).

Farmer John is observing Bessie's ballet drill and realizes that
sometimes she might not be able to jump to her destination lilypad
because intermediary lilypads are missing.

Ever thrifty, he wants to place additional lilypads so she can
complete her quest (perhaps quickly, perhaps by using a large number
of intermediate lilypads). Of course, lilypads cannot be placed
where rocks already intrude on a cell.

Help Farmer John determine the minimum number of additional lilypads
he has to place, and in how many ways he can place that minimum
number.

PROBLEM NAME: lilypad

INPUT FORMAT:

* Line 1: Two space-separated integers: M and N

* Lines 2..M+1: Line i+1 describes row i of the pond using N
        space-separated integers with these values: 0 indicates empty
        water; 1 indicates a lilypad in place; 2 indicates rock in
        place; 3 indicates the lilypad Bessie starts on; 4 indicates
        the lilypad Bessie wants to travel to.

SAMPLE INPUT (file lilypad.in):

4 5
1 0 0 0 0
3 0 0 0 0
0 0 2 0 0
0 0 0 4 0

INPUT DETAILS:

The ponds has 4 rows and 5 columns. Bessie is at row 2 column 1, and she
wants to reach row 4 column 4. A rock and three lilypads are present.

OUTPUT FORMAT:

* Line 1: One integer: the minimum number of additional lilypads
        required. If it is not possible to help Bessie jump to her
        destination, print -1.

* Line 2: One integer: the total number of possible ways the
        additional lilypads can be positioned.  This number is
        guaranteed to fit into a single 64-bit signed integer. Do not
        output this line if line 1 contains -1.

SAMPLE OUTPUT (file lilypad.out):

2
3

OUTPUT DETAILS:

Two lilypads are required. There are three ways to place them: row
4 column 2 and row 2 column 3; row 1 column 3 and row 3 column 2;
or row 1 column 3 and row 2 column 5:

          R1C2,R2C3     R1C3,R3C2     R1C3,R2C5
          1 0 0 0 0     1 0 X 0 0     1 0 X 0 0
          3 0 X 0 0     3 0 0 0 0     3 0 0 0 X
          0 0 2 0 0     0 X 2 0 0     0 0 2 0 0
          0 X 0 4 0     0 0 0 4 0     0 0 0 4 0
*/
// find path length (dijsktra) and prev[] vector for each lily position (empty and non-empty)
// then find DAG for to-put lilies only (i.e. for each empty lily, connect it (with a reverse edge, of course) to the empty lilies it can reach going backwards in the dijkstra prev[]-graph)
// then DP in the new graph to find count
// error: first solution found the number of distinct paths with min num of empty positions, not the ways new lilies can be put
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <ctime>
#include <cmath>
#include <vector>
#include <bitset>
#include <set>
#include <queue>
#include <map>
#include <utility>
#include <algorithm>
#include <functional>
#include <numeric>

using namespace std;

#define Repi(n) for (int i=0;i<n;i++)
#define Repj(n) for (int j=0;j<n;j++)
#define Repk(n) for (int k=0;k<n;k++)

#define PI pair<int,int>
#define PB push_back
#define VI vector<int>
#define MP make_pair
#define x first
#define F(i,n) for (int i=0;i<n;i++)
#define y second
#define PD pair<double,double>
#define LL long long
#define SZ size()
#define PQ priority_queue
#define sign(x) (((x)>0)-((x)<0)))

#define MAXN 32
int N,M;
int a[MAXN][MAXN];

PI mv[8];
typedef pair<int,PI> tip;
priority_queue<tip,vector<tip>,greater<tip> > q;
LL cnt[MAXN][MAXN];
int lev[MAXN][MAXN];
bool used[MAXN][MAXN];
vector<PI> prev[MAXN][MAXN],from[MAXN][MAXN];
PI init,fini;

bitset<MAXN> uu[MAXN][MAXN][MAXN];
void reach(int x,int y,int x1,int y1)
{
	if (uu[x][y][x1][y1]) return;
	uu[x][y][x1][y1]=1;
	if ((!a[x1][y1] || init==MP(x1,y1)) && (x1!=x || y1!=y))
	 {
			from[x][y].PB(MP(x1,y1));
			if (init==MP(x1,y1)) return;
			reach(x1,y1,x1,y1);
			return;
	 }
	Repi(prev[x1][y1].SZ)
			reach(x,y,prev[x1][y1][i].x,prev[x1][y1][i].y);
}



LL solve(int x,int y)
{
	LL &ret=cnt[x][y];
	if (ret>-1) return ret;
	ret=0;
	Repi(from[x][y].SZ)
	   ret+=solve(from[x][y][i].x,from[x][y][i].y);
	return ret;
}

int main()
{
	freopen("lilypad.in","r",stdin); freopen("lilypad.out","w",stdout);
	scanf("%d%d",&N,&M);
	mv[0]=MP(1,2);
	mv[1]=MP(-1,-2);
	mv[2]=MP(1,-2);
	mv[3]=MP(-1,2);
	mv[4]=MP(2,1);
	mv[5]=MP(-2,-1);
	mv[6]=MP(2,-1);
	mv[7]=MP(-2,1);
	
	memset(lev,-1,sizeof(lev));
	Repi(N)
	 Repj(M)
	  {
			scanf("%d",&a[i][j]);
			if (a[i][j]==3) init=MP(i,j);
			if (a[i][j]==4) fini=MP(i,j);
	  }

	q.push(MP(0,init));
	lev[init.x][init.y]=0;
	int x,y;
	while (!q.empty())
	 {
			x=q.top().y.x,y=q.top().y.y; q.pop();
			if (used[x][y]) continue;
			used[x][y]=1;
			Repi(8)
			 {
					int nx=x+mv[i].x,ny=y+mv[i].y;
					if (nx<0||nx>=N||ny<0||ny>=M) continue;
					if (a[nx][ny]==2) continue;
					
					if (lev[nx][ny]==-1)
					 {
							lev[nx][ny]=lev[x][y]+(!a[nx][ny]);
							q.push(MP(lev[nx][ny],MP(nx,ny)));
							prev[nx][ny].PB(MP(x,y));
					 }
					else
					 if (lev[nx][ny]==lev[x][y]+(!a[nx][ny]))
							prev[nx][ny].PB(MP(x,y));
			 }
	 }
	if (!used[fini.x][fini.y])
	 {			printf("-1\n"); return 0; 	  }
	
	memset(used,0,sizeof(used));
	reach(fini.x,fini.y,fini.x,fini.y);
	memset(cnt,-1,sizeof(cnt));
	cnt[init.x][init.y]=1;
	
	cout<<lev[fini.x][fini.y]<<"\n"<<solve(fini.x,fini.y)<<"\n";
	return 0;
}

