/*
PROG:robots
LANG:C++
ID:pascala1

Problem robots: Amazing Robots [IOI, 2003]

Memory Limit: 18MB due to system change.

You are the proud owner of two robots that are located in separate
rectangular mazes each of which has no more than 20 rows or columns.
Square (1, 1) in a maze is the square in the upper-left (northwest)
corner.  Maze i (i = 1, 2) has a set of G_i (0 <= G_i <= 10) guards
trying to capture the robots. They patrol back and forth on a
straight patrol path.

Your goal is to determine a sequence of robot commands such that
both robots exit the mazes without either robot being captured by
a guard.

Each robot command is a direction (north, south, east, or west).
At the beginning of each minute, you broadcast the same command to
both robots. A robot moves one square in the direction of the
command, unless the robot would collide with a wall, in which case
the robot does not move for that minute. A robot exits the maze by
walking out of it. A robot ignores commands after it has exited its
maze.

Guards move one square at the beginning of each minute, at the same
time as the robots. Each guard begins at a given square facing a
given direction and moves forward one square per minute until the
guard has moved one fewer square than the number of squares in his
patrol path. The guard then turns around instantaneously and walks
in the opposite direction back to his starting square, where he
again turns around and repeats his patrol path until each robot has
exited its maze. A guard's patrol will not require the guard to
walk through walls or exit the maze.

Although guard patrols may overlap, no two guards will ever collide:
they will never occupy the same square at the end of a minute, and
they will not exchange squares with each other during a minute. A
guard in a maze will not start in the same square as the robot in
that maze.  A guard captures a robot if the guard occupies the same
square at the end of a minute as the robot, or if the guard and the
robot exchange squares with each other during a minute.

Given the layout of the two mazes along with the initial square of
each robot and the patrol paths of the guards in each maze, determine
a sequence of commands for which the robots exit without being
caught by the guards. Minimize the time it takes for the later robot
to exit its maze. If the robots exit at different times, the time
at which the earlier robot exited does not matter.

TIME LIMIT: 5 seconds

PROBLEM NAME: robots

INPUT FORMAT:

* Line 1: Two space-separated integers: R1 and C1, the the number of
        rows and columns of the first maze

* Lines 2..R1+1: C1 characters each specifying the maze layout. The
        robot's starting square is specified by an `X'. A `.'
        represents an open space and `#' represents a wall. Each maze
        will contain exactly one robot.

* Line R1+2: A single integer: G_1

* Lines R1+3..R1+G1+2: Line i+R1+G1+1 contains three space-separated
        integers and a character describing guard i's: row, column,
        guard path length (2 <= length <= 4), and the initial
        direction the guard faces (`N', `S', `E', or `W'; north,
        south, east, and west, respectively).

* Lines R1+G1+3..end of input: Another set of lines formatted the same
        way, the description of the second maze.

SAMPLE INPUT (file robots.in):

5 4
####
#X.#
#..#
...#
##.#
1
4 3 2 W
4 4
####
#...
#X.#
####
0

OUTPUT FORMAT:

* Line 1: A single integer K (K <= 10000), the number of commands for
        both robots to exit the maze without being captured. If no
        such sequence exists, output a single line containing ``-1''.

SAMPLE OUTPUT (file robots.out):

8

OUTPUT DETAILS:

E N E S S S E S will do the trick; nothing shorter will.
*/
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <ctime>
#include <cmath>
#include <vector>
#include <bitset>
#include <set>
#include <queue>
#include <map>
#include <utility>
#include <algorithm>
#include <functional>
#include <numeric>

using namespace std;

#define Repi(n) for (int i=0;i<n;i++)
#define Repj(n) for (int j=0;j<n;j++)
#define Repk(n) for (int k=0;k<n;k++)

#define PI pair<int,int>
#define PB push_back
#define VI vector<int>
#define MP make_pair
#define x first
#define F(i,n) for (int i=0;i<n;i++)
#define y second
#define PD pair<double,double>
#define LL long long
#define SZ size()
#define PQ priority_queue
#define sign(x) (((x)>0)-((x)<0)))

#define MAXN 22
#define MAXG 13
const int MAXSTATE=MAXN*MAXN*MAXN*MAXN*MAXG;

PI mv[128];
int dirs[128];

struct guard{
	PI init;
	int len;
	int dir;
	PI pos[12];
	};

int N[2],G[2],M[2];
char maze[2][MAXN][MAXN];
PI init[2];
guard g[2][12];
int x[2],y[2],tim;

void getguardpos(guard &g)
{
	int x=g.init.x,y=g.init.y; int d=g.dir;
	for (int i=0,cl=0;i<12;i++,cl++)
	 {
			g.pos[i]=MP(x,y);
			if (cl==g.len-1) cl=0,d+=2,d%=4;
			x+=mv[d].x,y+=mv[d].y;
	 }
}

void read(int rob)
{
	scanf("%d%d",&N[rob],&M[rob]);
	Repi(N[rob])
	 {
	  scanf("%s",maze[rob][i]);
//	  cout<<maze[rob][i]<<endl;
	  Repj(M[rob])
	   if (maze[rob][i][j]=='X') x[rob]=i,y[rob]=j;
	 }
//	cout<<"    robot at "<<x[rob]<<" "<<y[rob]<<endl;
	scanf("%d",G+rob);
	Repi(G[rob])
	 {
			char c;
			scanf("%d%d%d %c",&g[rob][i].init.x,&g[rob][i].init.y,&g[rob][i].len,&c);
			g[rob][i].init.x--;
			g[rob][i].init.y--;
			g[rob][i].dir=dirs[c];
		//	cout<<"  guard "<<i<<" : "<<g[rob][i].init.x<<" "<<g[rob][i].init.y<<" "<<g[rob][i].len<<" "<<g[rob][i].dir<<endl;
   	        getguardpos(g[rob][i]);
   	      //  cout<<"       pos: \n";
   	      //  Repj(12)
   	      //   cout<<"                                   "<<g[rob][i].pos[j].x<<" "<<g[rob][i].pos[j].y<<endl;
	 }
}

int encode(int x1,int y1,int x2,int y2,int tm)
{
	return (((x1*MAXN+y1)*MAXN+x2)*MAXN+y2)*MAXG+tm;
}

void decode(int ret,int &x1,int &y1,int &x2,int &y2,int &tm)
{
	tm=ret%MAXG;
	ret/=MAXG; y2=ret%MAXN;
	ret/=MAXN; x2=ret%MAXN;
	ret/=MAXN; y1=ret%MAXN;
	ret/=MAXN; x1=ret;
}

const int MAXQ=MAXSTATE/MAXN;
int q[MAXQ],st,en;
short lev[MAXSTATE];

bool ok(int r,int x,int y,int t,int &xn,int &yn,int tn)
{
	if (xn<0 || xn>=N[r] || yn<0 || yn>=M[r]) xn=N[r],yn=M[r];
	if (maze[r][xn][yn]=='#') xn=x,yn=y;
//	cout<<"                                                ok? "<<r<<" ; "<<x<<" "<<y<<" "<<t<<" ; "<<xn<<" "<<yn<<" "<<tn<<"\n";
	Repi(G[r])
	 {
			if (g[r][i].pos[tn]==MP(xn,yn))
			 {
				//	cout<<"                                                      NOT !  guard "<<i<<" case 1\n";
					return 0;
			 }
			if (g[r][i].pos[tn]==MP(x,y) && g[r][i].pos[t]==MP(xn,yn))
			 {
				//	cout<<"                                                      NOT !  guard "<<i<<"  case 2\n";
					return 0;
			 }
	 }
	return 1;
}

int main()
{
	freopen("robots.in","r",stdin); freopen("robots.out","w",stdout);
	mv[0]=MP(-1,0);
	mv[1]=MP(0,-1);
	mv[2]=MP(1,0);
	mv[3]=MP(0,1);
	dirs['N']=0,dirs['W']=1,dirs['S']=2,dirs['E']=3;
	read(0);
	read(1);
	
	memset(lev,-1,sizeof(lev));
	int state=encode(x[0],y[0],x[1],y[1],0);
	q[0]=state;
	lev[state]=0;
	int ans=-1;
//	cout<<"\n\n\n\n";
	for (st=0,en=1;st!=en;st++)
	 {
			if (st>=MAXQ) st=0;
			decode(q[st],x[0],y[0],x[1],y[1],tim);
		//	cout<<"     at "<<x[0]<<" "<<y[0]<<" , "<<x[1]<<" "<<y[1]<<" , "<<tim<<" with "<<lev[q[st]]<<endl;
			if (lev[q[st]]==10000) goto finish;
			Repi(4)
			 {
				int ntim=tim+1; ntim%=12;
				int x0=x[0]+mv[i].x*(x[0]!=N[0]),y0=y[0]+mv[i].y*(y[0]!=M[0]);
				int x1=x[1]+mv[i].x*(x[1]!=N[1]),y1=y[1]+mv[i].y*(y[1]!=M[1]);
			//	cout<<"        trying to "<<x0<<" "<<y0<<" "<<x1<<" "<<y1<<" ("<<i<<")...\n";
				if (ok(0,x[0],y[0],tim,x0,y0,ntim) && ok(1,x[1],y[1],tim,x1,y1,ntim))
				 {
						state=encode(x0,y0,x1,y1,ntim);
						if (lev[state]>-1) continue;
						lev[state]=lev[q[st]]+1;
						q[en++]=state;
						if (en>=MAXQ) en=0;
				//		cout<<"              OK!!!!!!!!!!!!\n";
						if (x0==N[0] && y0==M[0] && x1==N[1] && y1==M[1]) { ans=lev[state]; goto finish; }
				 }
			 } 
	 }
	finish:;
	printf("%d\n",ans);
		
	return 0;
}

