/*
PROG:groc
LANG:C++
ID:pascala1

Problem groc: Grocery Store [Cox/Kolstad, 2006]

Bessie has gone to the grocery store with her list of P (1 <= P <=
300) items to purchase. This grocery store is laid out like so many
grocery stores with N aisles (1 <= N <= 350). Below is a diagram
showing not only the layout of a store with N=5 aisles but also the
location of seven items to purchase, the entrance, the length A of
each aisle (1 <= A <= 25), and the distance B from the center of
one isle to the center of the next aisle (1 <= B <= 5). The dots
represent the places Bessie might stop; many adjacent dots are
distance 1 apart.

img src=/store1a.gif

Bessie wants to minimize her walking distance to retrieve all the
items she wishes to purchase and then check out at the lower-right
corner.

Given A, B, and a list of P items to purchase (each item described
by its aisle number and distance from the 'beginning' of the aisle
[nearest the entrance]), calculate the minimum distance Bessie must
travel. Note that one step is required to enter a given vertical
aisle or leave a vertical aisle (to go to/from the right-left
traversal aisles). See the example below. Here is a sample route
(which might or might not be optimal) Bessie might consider:

img src=/store2a.gif

For the diagram above, A=10 (which means distance 11 from lower
aisle to upper aisle) and B=3. From the entrance to item 'a' is
length B + 8 = 11. 3 more steps to the upper cross-aisle (not
surprisingly a total of P+1=11 to traverse the entire aisle). 3*B=9
to aisle 5; 1 more to item f. Turning around, 1 more back to upper
cross-aisle, B=3 more to aisle 4, 1 down to get item 'e'. 1 back
up then B=3 left to aisle 3. Traverse 11 down aisle 3, picking up
'd', 'c', and 'b' along the way. B=3 right to Aisle 4. 3 up, get
item 'g', 3 down, B=3 right to the exit. Total = 56 steps.

PROBLEM NAME: groc

INPUT FORMAT:

* Line 1: Four space-separated integers: P, N, A, and B

* Lines 2..P+1: Each line contains two space-separated integers that
        describe the location of an item to buy, respectively the
        aisle number and distance from the 'beginning' of that aisle

SAMPLE INPUT (file groc.in):

7 5 10 3
2 8
3 3
3 5
3 7
4 10
5 10
4 3

OUTPUT FORMAT:

* Line 1: A single integer that is the smallest distance Bessie must
        travel to obtain all her items.

SAMPLE OUTPUT (file groc.out):

54
*/
// optimized dp n^4 (see groc_n4 for implementation)
// observation 1: the path has no cycles (it is always better to return rather than walk a closed rectangle)
// observation 2: main path always geos like this __|^^^|__|^^|_|^|______|^^^^^^|__... - you never repeat cells in this path
// obervation 2.1: there are some "branches" from the __ and ^^^ parts to take the groceries in the "empty" intervals - cells in them are always walked on twice
// solution: dp[aisle][up or down] = the min path to take all groceries left from aisle "aisle", walk the entire aisle and end up at its lower/upper end (0/1 respectively for the second parameter)
// to each [pos][0] we can go from any [i][1] (i<pos) in O(N^2) (analogically for [pos][1])
// we only need to find the best way to take the groceries between aisles pos and i
// this can be done in O(N^2) (it's simple to understand, but hard to explain) ;;; the zero and N+1-st aisles (dummy aisles) are corner cases, as well as the path ___________
// optimization 1: it is not optimal to come to aisle [pos] from an aisle before the one we've come from to aisle [pos-1]
// optimization 2: the second cycle in determining the cost for taking middle groceries can be done in O(1) using a static RMQ (the formula is rephrased - see clean N^4 implementation)
// optimization 3: it is not optimal to come from empty aisles (besides dummy aisle 0)  (this is not necessary to pass tests, but gives an improvement)
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <ctime>
#include <cmath>
#include <vector>
#include <bitset>
#include <set>
#include <queue>
#include <map>
#include <utility>
#include <algorithm>
#include <functional>
#include <numeric>

using namespace std;

#define Repi(n) for (int i=0;i<n;i++)
#define Repj(n) for (int j=0;j<n;j++)
#define Repk(n) for (int k=0;k<n;k++)

#define PI pair<int,int>
#define PB push_back
#define VI vector<int>
#define MP make_pair
#define x first
#define F(i,n) for (int i=0;i<n;i++)
#define y second
#define PD pair<double,double>
#define LL long long
#define SZ size()
#define PQ priority_queue
#define sign(x) (((x)>0)-((x)<0)))
#define INF 1000000000
#define MAXN 1024

int P,N,A,B;
PI g[MAXN];

int upperd[MAXN],lowerd[MAXN],bothd[MAXN];   //  for each aisle, the cost to take all groceries and return to initial position, if initial position is:
// for upper - in the upper end of the aisle
// for lower - in the lower end of the aisle
// in both - in both ends of the aisle (then they are split in an upper and lower half in an optimal way)
int *upper=upperd+1,*lower=lowerd+1,*both=bothd+1;  // to avoid upper[-1] etc
VI aisle[MAXN];        // rows with groceries in each aisle
int dp[MAXN][2];
int from[MAXN][2];   // optim 1


int rmq[2][11][MAXN];
void rmqprep(int whichone,int *upperorlower)
{
	Repj(N+2)
	 rmq[whichone][0][j]=upperorlower[j-1]*2 -both[j-1]*2 -j*2*B;
	for (int power=1,next=1;next<N+2;power++,next<<=1)
	 for (int j=0;j+next<N+2;j++)
	  rmq[whichone][power][j]=min(rmq[whichone][power-1][j],rmq[whichone][power-1][j+next]);
}


#define MAXN (1<<19)
int rmq[20][MAXN];
int a[MAXN],N;

void initialize()
{
	for (int i=0;i<N;i++)
	 rmq[0][i]=a[i];
	for (int pow=0,step=1; step<N; pow++,step<<=1)
	 for (int i=0; i+step<N; i++)
	  rmq[pow][i] = min( rmq[pow][i] , rmq[pow][i+step] );
}


int query(int which,int st,int en)
{
	int pow=31-__builtin_clz(en-st+1),step=1<<pow;
	return min(rmq[which][pow][st],rmq[which][pow][en-step+1]);
}




int main()
{
	freopen("groc.in","r",stdin); freopen("groc.out","w",stdout);
	scanf("%d%d%d%d",&P,&N,&A,&B);
	Repi(P)
	 scanf("%d%d",&g[i].x,&g[i].y),aisle[g[i].x].PB(g[i].y);
	sort(g,g+P);
	
	
	// initialize upper,lower,both arrays and RMQ
	for (int i=1;i<=N+1;i++)
	 {
			if (!aisle[i].SZ) goto xaxa;
			sort(aisle[i].begin(),aisle[i].end());
			
			upper[i]=A+1-aisle[i][0];
			lower[i]=aisle[i][aisle[i].SZ-1];
			both[i]=min(upper[i],lower[i]);
			Repj(aisle[i].SZ-1)
			 both[i]<?=aisle[i][j]+A+1-aisle[i][j+1];
			 
			xaxa:;
			both[i]+=both[i-1],upper[i]+=upper[i-1],lower[i]+=lower[i-1];
	 }
	rmqprep(0,upper);
	rmqprep(1,lower);
	 
	 
	//solve
	memset(dp,127,sizeof(dp));
	int cornercase=0;
	dp[0][0]=-B;
	for (int b=1;b<=N+1;b++)
	 {
			dp[b][1]=cornercase+A+1;
			cornercase+=(lower[b]-lower[b-1])*2;
  	        for (int a=b-1;a>=from[b-1][0] || a>=from[b-1][1];a--)
  	         if (aisle[a].SZ || !a)
  	          {
					int addd=(b-a)*B+A+1,tmp0=INF,tmp1=INF;
					for (int i=a;i<b && !((i>0)&&(a==0));i++)
					 {
							int add=(both[i]-both[a]+both[b-1])*2+(b+i-a)*2*B;
							tmp0<?=addd+add -upper[i]*2+query(0,(b==N+1)?(b):(i+1),b);
							tmp1<?=addd+add -lower[i]*2+query(1,(b==N+1)?(b):(i+1),b);
					 }
					if (dp[b][0]>dp[a][1]+tmp0)
					 dp[b][0]=dp[a][1]+tmp0,from[b][0]=a;
					if (dp[b][1]>dp[a][0]+tmp1)
					 dp[b][1]=dp[a][0]+tmp1,from[b][1]=a;
			  }
			cornercase+=B;
	 }


	int ans=cornercase-2*B;
	ans<?=dp[N][0];
	ans<?=dp[N+1][1]-A-1-B;
	printf("%d\n",ans);
	
	return 0;
}

