/*
TASK:rivers
LANG:C++
ID:pascala1
Problem rivers: Rivers [IOI, 2005]

Nearly all of the Kingdom of Byteland is covered by forests and
rivers. Small rivers meet to form bigger rivers, which also meet
and, in the end, all the rivers flow together into one big river.
The big river meets the sea near Bytetown.

There are N (2 <= N <= 100) lumberjack's villages (conveniently
number 1..N) in Byteland (which is village number 0), each placed
near a river. Currently, there is a big sawmill in Bytetown that
processes all trees cut in the Kingdom. The trees float from the
villages down the rivers to the sawmill in Bytetown. The king of
Byteland decided to build K (1 <= K <= 50; K <= N) additional
sawmills in villages to reduce the cost of transporting the trees
downriver. After building the sawmills, the trees need not float
to Bytetown, but can be processed in the first sawmill they encounter
downriver. Obviously, the trees cut near a village with a sawmill
need not be transported by river. It should be noted that the rivers
in Byteland do not fork. Therefore, for each village, there is a
unique way downriver from the village to Bytetown.

The king's accountants calculated how many trees are cut by each
village per year. You must decide where to build the sawmills to
minimize the total cost of transporting the trees per year. River
transportation costs one cent per kilometre, per tree.

Write a program that:

* Reads from the input file the number of villages, the number of
  additional sawmills to be built, the number of trees cut near
  each village, and descriptions of the rivers,

* Calculates the minimal cost of river transportation after building
  additional sawmills,

* Writes the result to the output file.

PROBLEM NAME: rivers

INPUT FORMAT:

* Line 1: Two integers: N and K

* Lines 2..N+1: Line i+1 contains three space-separated integers: W_i,
        V_i, and D_i.

* W_i: The number of trees cut near village i per year (0 <= W_i <= 10,000),

* V_i: The first village (or Bytetown) downriver from village i (0
       <= V_i <= N),

* D_i: The distance in kilometres by river from village i to V_i
       (1 <= D_i <= 10,000).

SAMPLE INPUT (file rivers.in):

4 2
1 0 1
1 1 10
10 2 5
1 2 3

OUTPUT FORMAT:

* Line 1: A single integer that is the minimal cost of river
        transportation in cents.

SAMPLE OUTPUT (file rivers.out):

4

OUTPUT DETAILS:

The sawmills should be built in villages 2 and 3
*/
// copied from my archives
#include <iostream>
#include <vector>
#include <cstdio>
#include <cstdlib>
#include <algorithm>

using namespace std;

#define Repi(n) for (int i=0;i<n;i++)
#define Repj(n) for (int j=0;j<n;j++)
#define Repk(n) for (int k=0;k<n;k++)
#define SZ size()
#define VI vector<int>
#define PI pair<int,int>
#define PB push_back
#define MP make_pair
#define x first
#define y second
#define LL long long
#define INF 10000000
#define MAXN 102
#define MAXK 52

int N,K;
int dd[MAXN],par[MAXN],trees[MAXN];
int dist[MAXN];
VI chi[MAXN];

int dp[MAXN][MAXN][MAXK]; //in, last, sawmills left
int cnt[MAXN][MAXN];
bool used[MAXN][MAXN];

void calcdist(int v)
{
	Repi(chi[v].SZ)
	 dist[chi[v][i]]=dist[v]+dd[chi[v][i]],calcdist(chi[v][i]);
}

int cost[2][MAXN],cc[2];
int getsums(int in,int last,int sums[]) //for each possible total number of sawmills used by in's children calculate the minimum total cost of "fixing" their subtrees, if the last placed sawmill above them is 'last'
{
	int Y=0,v,nsum;
	cc[0]=1; cost[0][0]=0;
	Repi(chi[in].SZ) //a.k.a. for each child of 'in'...
	 {
			v=chi[in][i];
			cc[Y^1]=0;
			Repj(::cnt[v][last]) //...for each possible number of mills that can be built in its subtree (when 'last' is previous mill)...
			 Repk(cc[Y]) //...for each possible total number of mills for its siblings to the left...
			  {
					nsum=k+j;
					if (nsum>K) break;
					if (nsum>=cc[Y^1]) { cost[Y^1][nsum]=INF; cc[Y^1]=nsum+1; }
					cost[Y^1][nsum]=min(cost[Y^1][nsum],cost[Y][k]+dp[v][last][j]); //...add min cost of new possible sum (with current child included)
			  }
			Y^=1;
	 }
   memcpy(sums,cost[Y],sizeof(int)*cc[Y]);
   return cc[Y];
}

int best2[MAXN],cnt2;
void solve(int in,int last)
{
	if (used[in][last]) return; used[in][last]=1;
	Repi(chi[in].SZ)
	 solve(chi[in][i],last),solve(chi[in][i],in);
	int *best=dp[in][last];
	int &cnt=::cnt[in][last];
	
	cnt=getsums(in,last,best); //find dp[in][last][i] for all 'i' if we haven't placed a sawmill in 'in'
	best[cnt++]=INF;
	Repi(cnt) best[i]+=(dist[in]-dist[last])*trees[in]; //since we only have the answer for the children, we have to add the cost of transporting the trees in 'in' to the last sawmill (since we haven't built one in it)
	getsums(in,in,best2); //find the same thing, only this time we've placed a sawmill in 'in'
	for (int i=1;i<cnt;i++)
	 best[i]=min(best[i],best2[i-1]); //the answer for each possible number of used sawmills is the minimum of both sequences (note that we substract one from the indices of the second since we assume we've built a sawmill in 'in')
}

int main()
{
	freopen("rivers.in","r",stdin); freopen("rivers.out","w",stdout);
	scanf("%d%d",&N,&K);
	Repi(N)
	 {
			scanf("%d%d%d",trees+i+1,par+i+1,dd+i+1);
			chi[par[i+1]].PB(i+1);
	 }
	dist[0]=0;
	calcdist(0);
	 
	memset(used,0,sizeof(used));
	solve(0,0);
	
	printf("%d\n",dp[0][0][K]);
	return 0;
} 
