/*
ID: pascala1
PROG: cowxor
LANG: C++
*/
//for each cow, all values = all values for the previous cow ^ current cow number , plus the current cow number as a new value
//to make our life easier, instead of performing the xor operation on all previous values for each cow (thus storing all values
//for the intervals [i,en] i=1..en at each iteration) we can keep the values for all intervals [1,i] i=1..en and 
//then represent the value of some arbitrary interval [st,en] as [1,en] ^ [1,st-1] (sort of like Sum(st,en)=sum[en]-sum[st-1] where sum[i]=sum of elements 1..i )
//but how does that help? well, if we store those values [1,i] in a bit-trie, we can find the maximal value for the current cow in O(21) time
//just the same as finding a maximal word in a trie, only that at each step we won't try to advance to the "highest" child (a.k.a. to add 1 to the bit representation when possible, otherwise add 0)
//but we will advance to the child which will have a higher value after performing a xor operation on it with the xor value of [1,en]
//see comments below for clarification
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <fstream>
#include <queue>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstdlib>
#include <ctime>


#define SZ size()
#define PB push_back
#define ALL(v) v.begin(),v.end()
#define MP make_pair
#define x first
#define y second

#define LL long long
#define UI unsigned int
#define ULL unsigned long long
#define PI pair<int,int>
#define PD pair<double,double>
#define PLL pair<LL,LL>
#define PULL pair<ULL,ULL>
#define VI vector<int>
#define VD vector<double>
#define VS vector<string>
#define SI set<int>
#define PQ priority_queue
#define VVI vector<vector<int> >
#define IT iterator

#define ABS(x) (((x)>0)?(x):(-(x)))
#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))
#define sign(a) ((a)>0)-((a)<0)
#define sqr(a) ((a)*(a))
#define Repi(n) for (int i=0; i<n; i++)
#define Repj(n) for (int j=0; j<n; j++)
#define Repk(n) for (int k=0; k<n; k++)

#define INF 2000000000
#define EPS 1e-6

#define Time ((double)clock()/CLOCKS_PER_SEC)
#define pause system("pause")

using namespace std;

int N;
const int MAXBIT=22;

struct bitnode{ bitnode *next[2]; int start; bitnode(){next[0]=next[1]=0; start=-1;}; };

bitnode *bitroot;

inline void add(int niu,int cowid) //simply add numbers bit-by-bit
{
	bitnode *cur=bitroot;
	bool to;
	for (int i=MAXBIT;i>=0;i--)
	 {
			to=(niu>>i)&1;
			if (!cur->next[to]) cur->next[to]=new bitnode;
			cur=cur->next[to];
	 }
	cur->start=cowid;
}

inline int findmax(int Xor,int &tmpst)
{
	int ret=0; bool to;
	bitnode *cur=bitroot;
	for (int i=MAXBIT;i>=0;i--)
	 {
			to=((Xor>>i)&1)^1; //if the i-th bit of xor-value [1,en] is 1, the i-th bit of [1,st-1] should best be 0 (for it will become 1 after being 'substracted' with the xor operation)
			                   //otherwise it should best be 1 (because it will then stay the same)
			ret<<=1;
			if (cur->next[to]) { ret|=to; cur=cur->next[to]; } //try to advance to the bit with higher priority
			else { ret|=to^1; cur=cur->next[to^1]; } //if we can't - go to the other...
	 }
	tmpst=cur->start; //get the position of the found number a.k.a. i for the value of [1,i] a.k.a. the start of the current optimal sequence
	return ret;
}

int main()
{
    freopen("cowxor.in","r",stdin);
    freopen("cowxor.out","w",stdout);

    int number,Xor=0,st,en,best=-1,tmp,tmpst;
    bitroot=new bitnode;
    scanf("%d",&N);
    Repi(N) 
     {
	    add(Xor,i+1); //add xor-value of all previous cows
	    scanf("%d",&number);
	    Xor^=number; //update xor-value
	    
	    tmp=findmax(Xor,tmpst);
	    tmp^=Xor;
	    if (tmp>best) best=tmp,st=tmpst,en=i+1; //update answer - optimality is guaranteed because we iterate cows in decreasing order of ranks (=>the first found with max value is the best) 
		                                        //and since we add to the trie every time, equal values are overwritten with the last cow they've been added with => the sequence will always be as short as possible
	 }

    printf("%d %d %d\n",best,st,en);
  
    return 0;
}
